package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.entity.BindAccountAuth;
import cn.com.ctop.common.module.service.IBindAccountAuthService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.StringUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @Description: 账号绑定-授权
 * @Author: jeecg-boot
 * @Date: 2019-07-31
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "账号绑定-授权")
@RestController
@RequestMapping("/ctop/bindAccountAuth")
public class BindAccountAuthController {
    @Autowired
    private IBindAccountAuthService bindAccountAuthService;

    /**
     * 参数校验
     *
     * @param json
     * @return
     */
    @PostMapping(value = "/authorization")
    public Result<IPage<BindAccountAuth>> authorization(@RequestBody JSONObject json) {
        log.info("授权入参:{}", json);
        Result<IPage<BindAccountAuth>> result = new Result<IPage<BindAccountAuth>>();
        String codeUrl = null;
        try {
            String advertiserId = json.getString("advertiserId");
            if (Check.isNull(advertiserId)) {
                result.setSuccess(false);
                return result;
            }

            String authorizationType = json.getString("authorizationType");

            if ("kuaishou".equals(authorizationType)) {
                codeUrl = bindAccountAuthService.getKuaishouCodeUrl(JSONObject.toJSONString(json));
            }
            if ("bytedance".equals(authorizationType)) {
                codeUrl = bindAccountAuthService.getByteDanceCodeUrl(JSONObject.toJSONString(json));
            }
            result.setMessage(codeUrl);
            result.setSuccess(true);
            return result;
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        result.setMessage(codeUrl);
        result.setSuccess(false);
        return result;

    }


    /**
     * 分页列表查询
     *
     * @param bindAccountAuth
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "账号绑定-授权-分页列表查询")
    @ApiOperation(value = "账号绑定-授权-分页列表查询", notes = "账号绑定-授权-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<BindAccountAuth>> queryPageList(BindAccountAuth bindAccountAuth,
                                                        @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                        @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                        HttpServletRequest req) {
        Result<IPage<BindAccountAuth>> result = new Result<IPage<BindAccountAuth>>();
        QueryWrapper<BindAccountAuth> queryWrapper = QueryGenerator.initQueryWrapper(bindAccountAuth, req.getParameterMap());
        Page<BindAccountAuth> page = new Page<BindAccountAuth>(pageNo, pageSize);
        IPage<BindAccountAuth> pageList = bindAccountAuthService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param bindAccountAuth
     * @return
     */
    @AutoLog(value = "账号绑定-授权-添加")
    @ApiOperation(value = "账号绑定-授权-添加", notes = "账号绑定-授权-添加")
    @PostMapping(value = "/add")
    public Result<BindAccountAuth> add(@RequestBody BindAccountAuth bindAccountAuth) {
        Result<BindAccountAuth> result = new Result<BindAccountAuth>();
        try {
            bindAccountAuthService.save(bindAccountAuth);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param bindAccountAuth
     * @return
     */
    @AutoLog(value = "账号绑定-授权-编辑")
    @ApiOperation(value = "账号绑定-授权-编辑", notes = "账号绑定-授权-编辑")
    @PutMapping(value = "/edit")
    public Result<BindAccountAuth> edit(@RequestBody BindAccountAuth bindAccountAuth) {
        Result<BindAccountAuth> result = new Result<BindAccountAuth>();
        BindAccountAuth bindAccountAuthEntity = bindAccountAuthService.getById(bindAccountAuth.getId());
        if (bindAccountAuthEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = bindAccountAuthService.updateById(bindAccountAuth);
            //TODO 返回false说明什么？
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "账号绑定-授权-通过id删除")
    @ApiOperation(value = "账号绑定-授权-通过id删除", notes = "账号绑定-授权-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            bindAccountAuthService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "账号绑定-授权-批量删除")
    @ApiOperation(value = "账号绑定-授权-批量删除", notes = "账号绑定-授权-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<BindAccountAuth> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<BindAccountAuth> result = new Result<BindAccountAuth>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.bindAccountAuthService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "账号绑定-授权-通过id查询")
    @ApiOperation(value = "账号绑定-授权-通过id查询", notes = "账号绑定-授权-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<BindAccountAuth> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<BindAccountAuth> result = new Result<BindAccountAuth>();
        BindAccountAuth bindAccountAuth = bindAccountAuthService.getById(id);
        if (bindAccountAuth == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(bindAccountAuth);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<BindAccountAuth> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                BindAccountAuth bindAccountAuth = JSON.parseObject(deString, BindAccountAuth.class);
                queryWrapper = QueryGenerator.initQueryWrapper(bindAccountAuth, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<BindAccountAuth> pageList = bindAccountAuthService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "账号绑定-授权列表");
        mv.addObject(NormalExcelConstants.CLASS, BindAccountAuth.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("账号绑定-授权列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<BindAccountAuth> listBindAccountAuths = ExcelImportUtil.importExcel(file.getInputStream(), BindAccountAuth.class, params);
                bindAccountAuthService.saveBatch(listBindAccountAuths);
                return Result.ok("文件导入成功！数据行数:" + listBindAccountAuths.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
