package org.jeecg.modules.ctop.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.ctop.entity.DramaItemInfo;
import org.jeecg.modules.ctop.service.IDramaItemInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 剧本内容信息
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-03-06
 */
@Slf4j
@Api(tags = "剧本内容信息")
@RestController
@RequestMapping("/ctop/dramaItemInfo")
public class DramaItemInfoController {
    @Autowired
    private IDramaItemInfoService dramaItemInfoService;

    /**
     * 分页列表查询
     *
     * @param dramaItemInfo
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "剧本内容信息-分页列表查询")
    @ApiOperation(value = "剧本内容信息-分页列表查询", notes = "剧本内容信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<DramaItemInfo>> queryPageList(DramaItemInfo dramaItemInfo,
                                                      @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                      @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                      HttpServletRequest req) {
        Result<IPage<DramaItemInfo>> result = new Result<IPage<DramaItemInfo>>();
        QueryWrapper<DramaItemInfo> queryWrapper = QueryGenerator.initQueryWrapper(dramaItemInfo, req.getParameterMap());
        Page<DramaItemInfo> page = new Page<DramaItemInfo>(pageNo, pageSize);
        IPage<DramaItemInfo> pageList = dramaItemInfoService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param dramaItemInfo
     * @return
     */
    @AutoLog(value = "剧本内容信息-添加")
    @ApiOperation(value = "剧本内容信息-添加", notes = "剧本内容信息-添加")
    @PostMapping(value = "/add")
    public Result<DramaItemInfo> add(@RequestBody DramaItemInfo dramaItemInfo) {
        Result<DramaItemInfo> result = new Result<>();
        try {
            String buttonType = dramaItemInfo.getButtonType();
            if (buttonType.startsWith("dialogue")) {
                dramaItemInfo.setType("dialogue");
                String[] infos = buttonType.split("_");
                dramaItemInfo.setRoleId(Long.parseLong(infos[1]));
            } else if (buttonType.startsWith("scence")) {
                dramaItemInfo.setType("scence");
            }
            dramaItemInfoService.save(dramaItemInfo);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param dramaItemInfo
     * @return
     */
    @AutoLog(value = "剧本内容信息-编辑")
    @ApiOperation(value = "剧本内容信息-编辑", notes = "剧本内容信息-编辑")
    @PostMapping(value = "/edit")
    public Result<DramaItemInfo> edit(@RequestBody DramaItemInfo dramaItemInfo) {
        Result<DramaItemInfo> result = new Result<>();
        DramaItemInfo dramaItemInfoEntity = dramaItemInfoService.getById(dramaItemInfo.getId());
        if (dramaItemInfoEntity == null) {
            result.error500("未找到对应实体");
        } else {
            String buttonType = dramaItemInfo.getButtonType();
            if (buttonType.startsWith("dialogue")) {
                dramaItemInfo.setType("dialogue");
                String[] infos = buttonType.split("_");
                dramaItemInfo.setRoleId(Long.parseLong(infos[1]));
            } else if (buttonType.startsWith("scence")) {
                dramaItemInfo.setType("scence");
            }
            boolean ok = dramaItemInfoService.updateById(dramaItemInfo);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @return
     */
    @AutoLog(value = "剧本内容信息-通过id删除")
    @ApiOperation(value = "剧本内容信息-通过id删除", notes = "剧本内容信息-通过id删除")
    @PostMapping(value = "/delete")
    public Result<?> delete(@RequestBody DramaItemInfo dramaItemInfo) {
        try {
            dramaItemInfoService.removeById(dramaItemInfo.getId());
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "剧本内容信息-批量删除")
    @ApiOperation(value = "剧本内容信息-批量删除", notes = "剧本内容信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<DramaItemInfo> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<DramaItemInfo> result = new Result<DramaItemInfo>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.dramaItemInfoService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "剧本内容信息-通过id查询")
    @ApiOperation(value = "剧本内容信息-通过id查询", notes = "剧本内容信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<DramaItemInfo> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<DramaItemInfo> result = new Result<DramaItemInfo>();
        DramaItemInfo dramaItemInfo = dramaItemInfoService.getById(id);
        if (dramaItemInfo == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(dramaItemInfo);
            result.setSuccess(true);
        }
        return result;
    }
}
