package org.jeecg.modules.ctop.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.ctop.entity.UReportFile;
import org.jeecg.modules.ctop.service.IUReportFileService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 报表文件存储
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-01-15
 */
@Slf4j
@Api(tags = "报表文件存储")
@RestController
@RequestMapping("/ctop/uReportFile")
public class UReportFileController {
    @Autowired
    private IUReportFileService uReportFileService;

    /**
     * 分页列表查询
     *
     * @param uReportFile
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "报表文件存储-分页列表查询")
    @ApiOperation(value = "报表文件存储-分页列表查询", notes = "报表文件存储-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<UReportFile>> queryPageList(UReportFile uReportFile,
                                                    @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                    @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                    HttpServletRequest req) {
        Result<IPage<UReportFile>> result = new Result<IPage<UReportFile>>();
        String name = null;
        if (null != uReportFile.getName() && !uReportFile.getName().trim().equals("")) {
            name = uReportFile.getName();
            uReportFile.setName(null);
        }
        QueryWrapper<UReportFile> queryWrapper = QueryGenerator.initQueryWrapper(uReportFile, req.getParameterMap());
        if (null != name) {
            queryWrapper.like("name", name);
        }
        Page<UReportFile> page = new Page<UReportFile>(pageNo, pageSize);
        IPage<UReportFile> pageList = uReportFileService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param uReportFile
     * @return
     */
    @AutoLog(value = "报表文件存储-添加")
    @ApiOperation(value = "报表文件存储-添加", notes = "报表文件存储-添加")
    @PostMapping(value = "/add")
    public Result<UReportFile> add(@RequestBody UReportFile uReportFile) {
        Result<UReportFile> result = new Result<UReportFile>();
        try {
            uReportFileService.save(uReportFile);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param uReportFile
     * @return
     */
    @AutoLog(value = "报表文件存储-编辑")
    @ApiOperation(value = "报表文件存储-编辑", notes = "报表文件存储-编辑")
    @PutMapping(value = "/edit")
    public Result<UReportFile> edit(@RequestBody UReportFile uReportFile) {
        Result<UReportFile> result = new Result<UReportFile>();
        UReportFile uReportFileEntity = uReportFileService.getById(uReportFile.getId());
        if (uReportFileEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = uReportFileService.updateById(uReportFile);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 绑定用户
     *
     * @return
     */
    @AutoLog(value = "报表文件绑定用户")
    @ApiOperation(value = "报表文件绑定用户", notes = "报表文件绑定用户")
    @PostMapping(value = "/bindUser")
    public Map<String, Object> bindUser(@RequestBody JSONObject data) {
        return uReportFileService.bindUsers(data);
    }

    @AutoLog(value = "报表文件解除绑定用户")
    @ApiOperation(value = "报表文件解除绑定用户", notes = "报表文件解除绑定用户")
    @PostMapping(value = "/unbindUser")
    public Map<String, Object> unbindUser(@RequestBody JSONObject data) {
        return uReportFileService.unbindUser(data);
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "报表文件存储-通过id删除")
    @ApiOperation(value = "报表文件存储-通过id删除", notes = "报表文件存储-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            uReportFileService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "报表文件存储-批量删除")
    @ApiOperation(value = "报表文件存储-批量删除", notes = "报表文件存储-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<UReportFile> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<UReportFile> result = new Result<UReportFile>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.uReportFileService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "报表文件存储-通过id查询")
    @ApiOperation(value = "报表文件存储-通过id查询", notes = "报表文件存储-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<UReportFile> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<UReportFile> result = new Result<UReportFile>();
        UReportFile uReportFile = uReportFileService.getById(id);
        if (uReportFile == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(uReportFile);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<UReportFile> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                UReportFile uReportFile = JSON.parseObject(deString, UReportFile.class);
                queryWrapper = QueryGenerator.initQueryWrapper(uReportFile, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<UReportFile> pageList = uReportFileService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "报表文件存储列表");
        mv.addObject(NormalExcelConstants.CLASS, UReportFile.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("报表文件存储列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<UReportFile> listUReportFiles = ExcelImportUtil.importExcel(file.getInputStream(), UReportFile.class, params);
                uReportFileService.saveBatch(listUReportFiles);
                return Result.ok("文件导入成功！数据行数:" + listUReportFiles.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }
}
