package org.jeecg.modules.ctop.service.impl;

import cn.com.ctop.common.module.entity.BindAccountLogin;
import cn.com.ctop.common.module.service.IBindAccountLoginService;
import cn.com.ctop.common.module.utils.HttpUtils;
import cn.com.ctop.common.module.utils.ResultMapUtils;
import cn.com.ctop.common.module.utils.StatusCode;
import cn.com.ctop.crawler.modules.appium.entity.AppiumDevice;
import cn.com.ctop.crawler.modules.appium.entity.AppiumTaskLog;
import cn.com.ctop.crawler.modules.appium.mapper.AppiumTaskLogMapper;
import cn.com.ctop.crawler.modules.appium.service.IAppiumDeviceService;
import cn.com.ctop.crawler.modules.appium.service.IAppiumJobService;
import cn.com.ctop.crawler.modules.appium.service.IAppiumTaskService;
import cn.com.ctop.crawler.modules.kuaishoucce.entity.KuaishouEffectAdInfo;
import cn.com.ctop.crawler.modules.kuaishoucce.entity.KuaishouHotPhotoInfo;
import cn.com.ctop.crawler.modules.kuaishoucce.entity.KuaishouInspiredAd;
import cn.com.ctop.crawler.modules.kuaishoucce.service.IKuaishouEffectAdInfoService;
import cn.com.ctop.crawler.modules.kuaishoucce.service.IKuaishouHotPhotoInfoService;
import cn.com.ctop.crawler.modules.kuaishoucce.service.IKuaishouInspiredAdService;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.modules.ctop.service.ICrawlerService;
import org.jeecg.modules.ctop.vo.KuaishouAppiumJobVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Service
@Slf4j
public class CrawlerServiceImpl implements ICrawlerService {
    static ExecutorService executorService = null;
    @Autowired
    private IBindAccountLoginService bindAccountLoginService;
    @Autowired
    private IAppiumJobService jobService;
    @Autowired
    private IAppiumTaskService taskService;
    @Autowired
    private IAppiumDeviceService deviceService;
    @Autowired
    private AppiumTaskLogMapper taskLogMapper;

    @Override
    public Map<String, Object> startJob(KuaishouAppiumJobVO jobVO) {
        Map<String, Object> result = new HashMap<>();
        //1:根据id查询快手账号信息
        BindAccountLogin accountLogin = bindAccountLoginService.getById(jobVO.getAccountId());
        if (null == accountLogin) {
            ResultMapUtils.setResultMap(result, StatusCode.COMMON_PARAM_ERROR.getCode());
            return result;
        }
        if (jobVO.getJobId() == null || jobVO.getJobId() == 0 || jobVO.getNum() == null || jobVO.getNum() == 0 || jobVO.getCreativeName() == null || "".equals(jobVO.getCreativeName()) || jobVO.getPublishName() == null || "".equals(jobVO.getPublishName().trim())) {
            ResultMapUtils.setResultMap(result, StatusCode.COMMON_PARAM_ERROR.getCode());
            return result;
        }
        Long num = jobVO.getNum();
        //获取手机设备信息
        AppiumDevice device = deviceService.getEnabled();
        if (null == device || device.getStatus() != 1) {
            ResultMapUtils.setResultMap(result, StatusCode.KUAISHOU_CRAWLER_APP_NO_ENABLED_DEVICE.getCode());
            return result;
        }
        executorService = Executors.newFixedThreadPool(2);
        executorService.submit(new Runnable() {
            @Override
            public void run() {
                int i = 0;
                while (i < num) {
                    if (i > num) {
                        break;
                    }
                    deviceService.updateById(device);
                    AppiumTaskLog log = new AppiumTaskLog();
                    log.setDeviceId(device.getId());
                    log.setDeviceIp(device.getIp());
                    log.setDevicePort(device.getPort());
                    log.setStatus(1 + "");
                    log.setTaskId(jobVO.getJobId());
                    log.setTaskName("快手刷量任务");
                    taskLogMapper.insert(log);
                    log.setId(log.getId());
                    try {
                        //一键新机
                        boolean success = jobService.refreshWebPhone();
                        if (!success) {
                            jobService.updateStatus(device, log, -4, 1);
                            continue;
                        }
                        Thread.sleep(10000L);
                        String ksId = jobService.loginTask(2L, device);
                        if (null == ksId) {
                            i++;
                            jobService.updateStatus(device, log, -1, 1);
                            continue;
                        }
                        boolean isBind = jobService.bindCreative(accountLogin.getAccountName(), accountLogin.getPassword(), jobVO.getPublishName(), ksId);
                        if (!isBind) {
                            i++;
                            jobService.updateStatus(device, log, -2, 1);
                            continue;
                        }
                        taskService.runTask(1L, device.getId(), jobVO.getPublishName());
                        jobService.updateStatus(device, log, 2, 1);
                        i++;
                    } catch (Exception e) {
                        jobService.updateStatus(device, log, -3, 1);
                        i++;
                        continue;
                    }
                }
            }
        });
        ResultMapUtils.setResultMap(result, StatusCode.KUAISHOU_CRAWLER_APP_JOB_StART.getCode());
        return result;
    }

    @Override
    public Map<String, Object> loadInspiration() {
        //(1):效果广告解析
        loadeffectAdByPage(100, 1);

        //(2):广告创意视频
        loadInspiredAdByPage(100, 1);

        //(3):快手热门作品
        loadHotPhotoByPage(100, 1);

        return null;
    }

    private void loadHotPhotoByPage(int pageSize, int pageNum) {
        String url = "https://cc.e.kuaishou.com/rest/creativeCentor/hotPhoto/list";
        JSONObject param = new JSONObject();
        param.put("pageNum", pageNum);
        param.put("pageSize", pageSize);
        param.put("sortType", 1);
        String result = HttpUtils.httpPostRequest(url, param, new HashMap<>());
        JSONObject resultObject = JSONObject.parseObject(result);
        if (null == resultObject.getLong("result") || resultObject.getLong("result") != 1) {
            log.info("热门作品数据获取失败");
            return;
        }
        JSONArray hotPhotoInfos = resultObject.getJSONObject("data").getJSONArray("hotPhotoInfos");
        if (null == hotPhotoInfos || hotPhotoInfos.size() <= 0) {
            log.info("热门作品数据获取完成");
            return;
        }
        List<KuaishouHotPhotoInfo> photoInfos = new ArrayList<>();
        for (int i = 0; i < hotPhotoInfos.size(); i++) {
            JSONObject photoInfoJSONObject = hotPhotoInfos.getJSONObject(i);
            KuaishouHotPhotoInfo hotPhotoInfo = new KuaishouHotPhotoInfo(photoInfoJSONObject);
            photoInfos.add(hotPhotoInfo);
        }
        hotPhotoInfoService.replaceBatch(photoInfos);
        loadHotPhotoByPage(pageSize, pageNum + 1);
    }

    private void loadInspiredAdByPage(int pageSize, int pageNumber) {
        String url = "https://cc.e.kuaishou.com/rest/creativeCentor/inspired/list";
        JSONObject param = new JSONObject();
        param.put("pageNum", pageNumber);
        param.put("pageSize", pageSize);
        param.put("firstIndustryId", 0);
        param.put("formatId", 0);
        param.put("inspiredSortTypeId", 0);
        param.put("platformSourceId", 0);
        param.put("queryCond", "");
        String result = HttpUtils.httpPostRequest(url, param, new HashMap<>());
        JSONObject resultObject = JSONObject.parseObject(result);
        if (null == resultObject.getLong("result") || resultObject.getLong("result") != 1) {
            log.info("广告创意数据获取失败");
            return;
        }
        JSONArray inspiredAds = resultObject.getJSONObject("data").getJSONArray("inspiredAds");
        if (null == inspiredAds || inspiredAds.size() <= 0) {
            log.info("广告创意数据获取完成");
            return;
        }
        List<KuaishouInspiredAd> ads = new ArrayList<>();
        for (int i = 0; i < inspiredAds.size(); i++) {
            JSONObject ad = inspiredAds.getJSONObject(i);
            KuaishouInspiredAd inspiredAd = new KuaishouInspiredAd(ad);
            ads.add(inspiredAd);
        }
        inspiredAdService.replaceBatch(ads);
        loadInspiredAdByPage(pageSize, pageNumber + 1);
    }

    @Autowired
    private IKuaishouInspiredAdService inspiredAdService;
    @Autowired
    private IKuaishouHotPhotoInfoService hotPhotoInfoService;
    @Autowired
    private IKuaishouEffectAdInfoService effectAdInfoService;

    private void loadeffectAdByPage(int pageSize, int pageNum) {
        String url = "https://cc.e.kuaishou.com/rest/creativeCentor/adCase/effectAd/list";
        JSONObject param = new JSONObject();
        param.put("caseType", 1);
        param.put("pageNum", pageNum);
        param.put("pageSize", pageSize);
        String result = HttpUtils.httpPostRequest(url, param, new HashMap<>());
        JSONObject resultObject = JSONObject.parseObject(result);
        if (null == resultObject.getLong("result") || resultObject.getLong("result") != 1) {
            log.info("效果广告数据获取失败");
            return;
        }
        JSONArray effectAds = resultObject.getJSONObject("data").getJSONArray("effectAds");
        if (null == effectAds || effectAds.size() <= 0) {
            log.info("效果广告数据获取完成");
            return;
        }
        List<KuaishouEffectAdInfo> adInfos = new ArrayList<>();
        for (int i = 0; i < effectAds.size(); i++) {
            JSONObject ad = effectAds.getJSONObject(i);
            Long contentId = ad.getLong("contentId");
            if (null != contentId) {
                JSONObject content = getEffectAdDetail(contentId).getJSONObject("data").getJSONObject("effectAdDetail");
                KuaishouEffectAdInfo adInfo = new KuaishouEffectAdInfo(content);
                adInfos.add(adInfo);
            }
        }
        if (adInfos.size() > 0) {
            effectAdInfoService.replaceBatch(adInfos);
        }
        loadeffectAdByPage(pageSize, pageNum + 1);
    }

    private JSONObject getEffectAdDetail(Long contentId) {
        String url = "https://cc.e.kuaishou.com/rest/creativeCentor/adCase/effectAd/detail";
        JSONObject param = new JSONObject();
        param.put("caseType", 1);
        param.put("contentId", contentId);
        String result = HttpUtils.httpPostRequest(url, param, new HashMap<>());
        return JSONObject.parseObject(result);
    }
}
