package org.jeecg.modules.system.service.impl;

import cn.com.ctop.common.module.entity.MaterialInfo;
import cn.com.ctop.performanceappraisal.entity.Performance;
import cn.com.ctop.performanceappraisal.mapper.PerformanceMapper;
import cn.com.ctop.planeprojectcost.entity.PlaneProjectCost;
import cn.com.ctop.planeprojectcost.mapper.PlaneProjectCostMapper;
import cn.com.ctop.userefficientvideomap.entity.*;
import cn.com.ctop.userefficientvideomap.mapper.UserEfficientVideoMapMapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.common.util.DateUtils;
import org.jeecg.modules.system.entity.EffiRateComparator;
import org.jeecg.modules.system.entity.UserDto;
import org.jeecg.modules.system.mapper.SysRoleMapper;
import org.jeecg.modules.system.service.IPerformanceService2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.ParseException;
import java.util.*;

/**
 * 设计师季度绩效表
 *
 * @author sunzhen
 * @version V1.0
 * @date 2019-12-10
 */
@Slf4j
@Service
public class PerformanceServiceImpl2 extends ServiceImpl<PerformanceMapper, Performance> implements IPerformanceService2 {

    @Autowired
    private PerformanceMapper performanceMapper;
    @Autowired
    private UserEfficientVideoMapMapper userEfficientVideoMapMapper;
    @Autowired
    private SysRoleMapper sysRoleMapper;
    @Autowired
    private PlaneProjectCostMapper planeProjectCostMapper;

    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //有效视频计算开始
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * 获取季度有效视频和角色对应关系的定时任务
     * 每天定时任务：角色对应本季度有效视频计算--需要工具类有效视频计算工具类
     * <p>
     * 本周五-下一个周四 为一周的计算起止时间
     * //例如：20191220-20191226这周
     * //总视频数：这一周时间内发布的所有视频的总数；
     * //有效视频数：20191129-20191205这一周发布的所有视频，在20191206-20191219这两周内成为有效视频的数量；
     * <p>
     * 1.有效视频应分平台，不同平台的人使用自己平台的计算（极少会有相同素材在两个平台上的交叉，这种情况下人工处理）
     * 2.设置周五为一个星期的第一天，则下一周的周四为一周的最后一天
     * 3.使用当前时间计算出周五的时间，然后根据此时间找出本周计算有效视频的时间，进行计算
     *
     * @param calcDateStr yyyy-MM-dd格式的日期
     */
    @Override
    public void efficientVideoTask(String calcDateStr) {
        Date calcDate = null;
        try {
            calcDate = DateUtils.parseDate(calcDateStr, "yyyy-MM-dd");
        } catch (ParseException e) {
            e.printStackTrace();
        }
        //传入时间，获取对应的年份
        Date friday = DateUtils.getFriday(calcDate);
        String effiCalcStart = DateUtils.formatDate(DateUtils.addDay(friday, -14)); //计算有效视频开始的时间
        log.info("有效视频统计的开始时间为" + effiCalcStart);
        String effiCalcEnd = DateUtils.formatDate(DateUtils.addDay(friday, -1)); //计算有效视频结束的时间
        log.info("有效视频统计的结束时间为" + effiCalcEnd);
        String effiStart = DateUtils.formatDate(DateUtils.addDay(friday, -21)); //有效视频发布开始时间
        log.info("有效视频发布的开始时间为" + effiStart);
        String effiEnd = DateUtils.formatDate(DateUtils.addDay(friday, -15)); //有效视频发布结束时间
        log.info("有效视频发布的结束时间为" + effiEnd);

        //以周为维度计算
        //获取当前时间所在周所在年份以及季度（如果季度末最后一周的天数>=4则这周属于此季度，否则属于下一个季度）
        Map<String, Integer> yearQuarterMap = DateUtils.getYearQuarter(calcDate);
        int thisYear = yearQuarterMap.get("year"); //获取当天所在周所在的年份
        int thisMonth = yearQuarterMap.get("month"); //获取当天所在周所在的月份
        int thisQuarter = yearQuarterMap.get("quarter"); //获取当天所在周所在的季度
        log.info("统计时间所在年" + thisYear + "月" + thisMonth + "季" + thisQuarter);

        //快手有效视频
        kuaishouEffiVideo(calcDate, effiCalcStart, effiCalcEnd, effiStart, effiEnd, thisYear, thisMonth, thisQuarter);
        //头条有效视频
        toutiaoEffiVideo(calcDate, effiCalcStart, effiCalcEnd, effiStart, effiEnd, thisYear, thisMonth, thisQuarter);
    }

    //快手有效视频统计
    private void kuaishouEffiVideo(Date calcDate, String effiCalcStart, String effiCalcEnd, String effiStart, String effiEnd, int thisYear, int thisMonth, int thisQuarter) {
        log.info("快手有效视频统计开始，统计时间为 " + calcDate);
        //获取时间段内所有<快手>的有效视频
        List<EffiVideoDTO> EffiVideoDTOList = userEfficientVideoMapMapper.getUserVideoMap(effiStart, effiEnd, effiCalcStart, effiCalcEnd);

        for (EffiVideoDTO effiVideoDTO : EffiVideoDTOList) {
            //判断此视频是否已经在关系表中存在，如果已经存在则说明此视频已经存在,跳过即可
            //有效视频的逻辑是无论是快手还是头条
            int signatureCount = userEfficientVideoMapMapper.getEffiVideoCountBySignature(effiVideoDTO.getSignature(), new Integer(2));//1快手 2头条
            if (signatureCount != 0) {
                continue;
            }

            UserEfficientVideoMap shot = new UserEfficientVideoMap();
            UserEfficientVideoMap plane = new UserEfficientVideoMap();
            UserEfficientVideoMap plan = new UserEfficientVideoMap();
            UserEfficientVideoMap clip = new UserEfficientVideoMap();

            //拍摄
            shot.setUserId(effiVideoDTO.getShotId());
            shot.setEfficientVideoSignature(effiVideoDTO.getSignature());
            shot.setQuarter(thisQuarter);
            shot.setAppType(2); //1头条 2快手
            shot.setYear(thisYear);
            shot.setMonth(thisMonth);
            shot.setCalcDate(calcDate);
            shot.setRoleId("7bff9afed625aeeabca6bffe3c189183");
            userEfficientVideoMapMapper.insert(shot);

            //平面
            plane.setUserId(effiVideoDTO.getPlaneId());
            plane.setEfficientVideoSignature(effiVideoDTO.getSignature());
            plane.setQuarter(thisQuarter);
            plane.setAppType(2); //1头条 2快手
            plane.setYear(thisYear);
            plane.setMonth(thisMonth);
            plane.setCalcDate(calcDate);
            plane.setRoleId("8dc30cb9810bde89bdc3fa8a85b830b0");
            userEfficientVideoMapMapper.insert(plane);

            //策划
            plan.setUserId(effiVideoDTO.getPlanId());
            plan.setEfficientVideoSignature(effiVideoDTO.getSignature());
            plan.setQuarter(thisQuarter);
            plan.setAppType(2); //1头条 2快手
            plan.setYear(thisYear);
            plan.setMonth(thisMonth);
            plan.setCalcDate(calcDate);
            plan.setRoleId("0214283aa16f943efbb149ea4bb18f18");
            userEfficientVideoMapMapper.insert(plan);

            //剪辑
            clip.setUserId(effiVideoDTO.getClipId());
            clip.setEfficientVideoSignature(effiVideoDTO.getSignature());
            clip.setQuarter(thisQuarter);
            clip.setAppType(2); //1头条 2快手
            clip.setYear(thisYear);
            clip.setMonth(thisMonth);
            clip.setCalcDate(calcDate);
            clip.setRoleId("f38d8d70cf7ec50d5357a749e4dbf8ee");
            userEfficientVideoMapMapper.insert(clip);

            //快手有效视频同步到有效视频素材库
            userEfficientVideoMapMapper.updateKuaishouEffiType(1, effiVideoDTO.getSignature());

        }
        log.info("快手有效视频统计结束，统计时间为 " + calcDate);
    }

    //头条有效视频计算
    private void toutiaoEffiVideo(Date calcDate, String effiCalcStart, String effiCalcEnd, String effiStart, String effiEnd, int thisYear, int thisMonth, int thisQuarter) {
        log.info("头条手有效视频统计开始，统计时间为 " + calcDate);
        //获取时间段内所有头条的有效视频
        List<EffiVideoDTO> EffiVideoDTOList = userEfficientVideoMapMapper.getToutiaoUserVideoMap(effiStart, effiEnd, effiCalcStart, effiCalcEnd);

        for (EffiVideoDTO effiVideoDTO : EffiVideoDTOList) {
            //判断此视频是否已经在关系表中存在，如果已经存在则说明此视频已经存在,跳过即可
            //有效视频的逻辑是无论是快手还是头条
            int signatureCount = userEfficientVideoMapMapper.getEffiVideoCountBySignature(effiVideoDTO.getSignature(), new Integer(1));//1头条 2快手
            if (signatureCount != 0) {
                continue;
            }

            UserEfficientVideoMap shot = new UserEfficientVideoMap();
            UserEfficientVideoMap plane = new UserEfficientVideoMap();
            UserEfficientVideoMap plan = new UserEfficientVideoMap();
            UserEfficientVideoMap clip = new UserEfficientVideoMap();

            //拍摄
            shot.setUserId(effiVideoDTO.getShotId());
            shot.setEfficientVideoSignature(effiVideoDTO.getSignature());
            shot.setQuarter(thisQuarter);
            shot.setAppType(1); //1头条 2快手
            shot.setYear(thisYear);
            shot.setMonth(thisMonth);
            shot.setCalcDate(calcDate);
            shot.setRoleId("7bff9afed625aeeabca6bffe3c189183");
            //role_code shot
            userEfficientVideoMapMapper.insert(shot);

            //平面
            plane.setUserId(effiVideoDTO.getPlaneId());
            plane.setEfficientVideoSignature(effiVideoDTO.getSignature());
            plane.setQuarter(thisQuarter);
            plane.setAppType(1); //1头条 2快手
            plane.setYear(thisYear);
            plane.setMonth(thisMonth);
            plane.setCalcDate(calcDate);
            plane.setRoleId("8dc30cb9810bde89bdc3fa8a85b830b0");
            //role_code shot
            userEfficientVideoMapMapper.insert(plane);

            //策划
            plan.setUserId(effiVideoDTO.getPlanId());
            plan.setEfficientVideoSignature(effiVideoDTO.getSignature());
            plan.setQuarter(thisQuarter);
            plan.setAppType(1); //1头条 2快手
            plan.setYear(thisYear);
            plan.setMonth(thisMonth);
            plan.setCalcDate(calcDate);
            plan.setRoleId("0214283aa16f943efbb149ea4bb18f18");
            //role_code plan
            userEfficientVideoMapMapper.insert(plan);

            //剪辑
            clip.setUserId(effiVideoDTO.getClipId());
            clip.setEfficientVideoSignature(effiVideoDTO.getSignature());
            clip.setQuarter(thisQuarter);
            clip.setAppType(1); //1头条 2快手
            clip.setYear(thisYear);
            clip.setMonth(thisMonth);
            clip.setCalcDate(calcDate);
            clip.setRoleId("f38d8d70cf7ec50d5357a749e4dbf8ee");
            //role_code clip
            userEfficientVideoMapMapper.insert(clip);

            //头条有效视频同步到有效视频素材库
            userEfficientVideoMapMapper.updateToutiaoEffiType(1, effiVideoDTO.getSignature());
        }
        log.info("头条有效视频统计结束，统计时间为 " + calcDate);
    }
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //有效视频计算结束
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //绩效计算开始
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /**
     * 分平台：快手、抖音
     *
     * 编导、剪辑、拍摄：
     * 视频有效率 = 有效视频数/总视频数
     * 快手平台的设计师根据角色计算每一类角色的有效率排名，抖音平台的设计师根据角色计算每一类角色的有效率排名。两种平台分别计算其下员工绩效。
     * 根据平台和有效率排名找出提点比例，根据角色找出提成比例，计算绩效。
     *
     * 平面设计：
     * 设计师季度绩效 = （本季度所在项目时间内）所有[项目]的消耗 * 5%
     * 同一个项目同时存在多个平面的情况下，所有平面设计师平分此消耗
     *
     * 设计师leader：
     * leader小组下所有[剪辑]的季度[素材]消耗总合 * 平台比例 * 20%
     * 设计师leader不需要计算媒体总任务
     */
    //季度绩效任务（在有效视频绑定人定时任务之后跑）：设计师绩效计算-----包括快手和头条的计算逻辑
    @Override
    public void designerPerformanceTask(int year, int quarter) {
        log.info(year + "年第" + quarter + "季度快手设计师季度绩效计算开始");

        kuaishouDesigner(year, quarter);//快手
        toutiaoDesigner(year, quarter);//头条

        log.info(year + "年第" + quarter + "快手设计师季度绩效计算结束");
    }

    //快手设计师、设计师leader和平面设计师逻辑
    @Override
    public void kuaishouDesigner(int thisYear, int thisQuarter) {
        log.info(thisYear + "年第" + thisQuarter + "季度快手设计师季度绩效计算开始");
        Map<String, String> quarterStartEndDateMap = DateUtils.quarterStartEndDate(thisYear, thisQuarter);
        String startTime = quarterStartEndDateMap.get("startDate");
        String endTime = quarterStartEndDateMap.get("endDate");

        //编导
        kuaishouPlanPerformance(thisYear, thisQuarter, startTime, endTime);
        //拍摄
        kuaishouShotPerformance(thisYear, thisQuarter, startTime, endTime);
        //剪辑
        kuaishouClipPerformance(thisYear, thisQuarter, startTime, endTime);
        //平面设计
        kuaishouPlanePerformance(thisYear, thisQuarter, startTime, endTime);
        //设计师leader
        kuaishouDesignLeaderPerformance(thisYear, thisQuarter, startTime, endTime);

        log.info(thisYear + "年第" + thisQuarter + "季度快手设计师季度绩效计算开始");
    }

    //头条设计师逻辑
    @Override
    public void toutiaoDesigner(int thisYear, int thisQuarter) {
        log.info(thisYear + "年第" + thisQuarter + "季度头条设计师季度绩效计算开始");
        Map<String, String> quarterStartEndDateMap = DateUtils.quarterStartEndDate(thisYear, thisQuarter);
        String startTime = quarterStartEndDateMap.get("startDate");
        String endTime = quarterStartEndDateMap.get("endDate");

        //编导
        toutiaoPlanPerformance(thisYear, thisQuarter, startTime, endTime);
        //拍摄
        toutiaoShotPerformance(thisYear, thisQuarter, startTime, endTime);
        //剪辑
        toutiaoClipPerformance(thisYear, thisQuarter, startTime, endTime);
        //平面设计
        toutiaoPlanePerformance(thisYear, thisQuarter, startTime, endTime);
        //设计师leader
        toutiaoDesignLeaderPerformance(thisYear, thisQuarter, startTime, endTime);

        log.info(thisYear + "年第" + thisQuarter + "头条设计师季度绩效计算开始");
    }

    ////////////////////////////////////////////////////////////////////////////////////////////
    //快手
    ////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////////编导季度绩效计算start
    private void kuaishouPlanPerformance(int thisYear, int thisQuarter, String startTime, String endTime) {
        log.info(thisYear + "年第" + thisQuarter + "季度快手编导季度绩效计算开始");
        //1.根据角色（编导）查询出相对应角色所有的人
        //List<UserDto> planList = sysRoleMapper.getUserByRoleCode("plan"); //编导
        List<UserDto2> planList = userEfficientVideoMapMapper.getKuaishouPlanList(); //编导
        //循环编导列表计算视频有效率并排序
        for (UserDto2 userDto : planList) {
            //有效视频数量 1toutiao 2kuaishou
            int effiVideoCount = userEfficientVideoMapMapper.getEffiVideoCountByUserId(2, thisYear, thisQuarter, userDto.getUserId(), "0214283aa16f943efbb149ea4bb18f18");
            //总视频数量
            int videoCount = userEfficientVideoMapMapper.getQuarterMaterialCountByUserId("plan", startTime, endTime, userDto.getUserId());
            //视频有效率
            BigDecimal effiRate = new BigDecimal(String.valueOf(effiVideoCount)).divide(new BigDecimal(String.valueOf(videoCount)));
            userDto.setEffiRate(effiRate);
        }
        Collections.sort(planList, new EffiRateComparator());

        //获取编导人数并按照提点比例计算每个梯度的人数
        int planLength = planList.size();
        log.info("快手参与有效率排名的编导总人数为：" + planLength);
        //获取前30%和中间50%的人数（视频有效率四舍五入）
        int top30 = new BigDecimal(String.valueOf(planLength)).multiply(new BigDecimal("0.3")).setScale(0, BigDecimal.ROUND_HALF_UP).intValue();
        log.info("快手参与有效率排名的编导前30%人数为：" + top30);
        int middle50 = new BigDecimal(String.valueOf(planLength)).multiply(new BigDecimal("0.5")).setScale(0, BigDecimal.ROUND_HALF_UP).intValue();
        log.info("快手参与有效率排名的编导中间50%人数为：" + middle50);
        //测试数据
        // top30 = 1;
        // middle50 =1;

        //循环前30%有绩效的人数并计算其绩效，并入库
        for (int i = 0; i <= top30 - 1; i++) {
            //总消耗--获取这个人的总消耗
            BigDecimal totalCost = userEfficientVideoMapMapper.getTotalCostByPlanId(planList.get(i).getUserId(), startTime, endTime);
            //总绩效=总消耗*提点比例（5‰）*提成比例（编导40%）
            BigDecimal totalPerformance = totalCost.multiply(new BigDecimal("0.005")).multiply(new BigDecimal("0.4"));

            Performance performance = new Performance();
            performance.setAppType(2); //1头条 2快手
            performance.setRoleId(planList.get(i).getRoleId());
            performance.setQuarter(thisQuarter);
            performance.setYear(thisYear);
            performance.setCommissionRate(new BigDecimal("0.005"));
            performance.setRoleCommissionRate(new BigDecimal("0.4"));
            performance.setTotalCost(totalCost);
            performance.setTotalPerformance(totalPerformance);
            performance.setStartTime(startTime);
            performance.setEndTime(endTime);
            performance.setVideoEfficiency(planList.get(i).getEffiRate().setScale(4, BigDecimal.ROUND_HALF_UP));
            performance.setUserId(planList.get(i).getUserId());
            performanceMapper.insert(performance);
        }

        //去掉list中前30的数据
        planList.subList(0, top30 - 1).clear();

        //循环中间50%有绩效的人，计算绩效并入库
        for (int i = 0; i <= middle50 - 1; i++) {
            //总消耗--获取这个人所在项目的总消耗
            BigDecimal totalCost = userEfficientVideoMapMapper.getTotalCostByPlanId(planList.get(i).getUserId(), startTime, endTime);
            //总绩效=总消耗*提点比例（3‰）*提成比例（编导40%）
            BigDecimal totalPerformance = totalCost.multiply(new BigDecimal("0.003")).multiply(new BigDecimal("0.4"));

            Performance performance = new Performance();
            performance.setAppType(2); //1头条 2快手
            performance.setRoleId(planList.get(i).getRoleId());
            performance.setQuarter(thisQuarter);
            performance.setYear(thisYear);
            performance.setCommissionRate(new BigDecimal("0.003"));
            performance.setRoleCommissionRate(new BigDecimal("0.4"));
            performance.setTotalCost(totalCost);
            performance.setTotalPerformance(totalPerformance);
            performance.setStartTime(startTime);
            performance.setEndTime(endTime);
            performance.setVideoEfficiency(planList.get(i).getEffiRate().setScale(4, BigDecimal.ROUND_HALF_UP));
            performance.setUserId(planList.get(i).getUserId());
            performanceMapper.insert(performance);
        }
        log.info(thisYear + "年第" + thisQuarter + "季度快手设计师leader季度绩效计算结束");
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////快手编导绩效计算end

    ////////////////////////////////////////////////////////////////////////////////////////////////////快手拍摄绩效计算start
    private void kuaishouShotPerformance(int thisYear, int thisQuarter, String startTime, String endTime) {
        log.info(thisYear + "年第" + thisQuarter + "季度快手拍摄季度绩效计算开始");
        //根据角色（拍摄）查询出相对应角色所有的人
        //List<UserDto> shotList = sysRoleMapper.getUserByRoleCode("shot"); //拍摄
        List<UserDto2> shotList = userEfficientVideoMapMapper.getKuaishouShotList(); //拍摄
        for (UserDto2 userDto : shotList) {
            //有效视频数量 //1toutiao 2kuaishou
            int effiVideoCount = userEfficientVideoMapMapper.getEffiVideoCountByUserId(2, thisYear, thisQuarter, userDto.getUserId(), "7bff9afed625aeeabca6bffe3c189183");
            //总视频数量
            int videoCount = userEfficientVideoMapMapper.getQuarterMaterialCountByUserId("shot", startTime, endTime, userDto.getUserId());
            //视频有效率
            BigDecimal effiRate = new BigDecimal(String.valueOf(effiVideoCount)).divide(new BigDecimal(String.valueOf(videoCount)));
            userDto.setEffiRate(effiRate);
        }
        Collections.sort(shotList, new EffiRateComparator());

        //获取编导人数并按照提点比例计算每个梯度的人数
        int shotLength = shotList.size();
        log.info("快手参与有效率排名的拍摄总人数为：" + shotLength);
        //获取前30%和中间50%的人数（视频有效率四舍五入）
        int shotTop30 = new BigDecimal(String.valueOf(shotLength)).multiply(new BigDecimal("0.3")).setScale(0, BigDecimal.ROUND_HALF_UP).intValue();
        log.info("快手参与有效率排名的拍摄前30%人数为：" + shotTop30);
        int shotMiddle50 = new BigDecimal(String.valueOf(shotLength)).multiply(new BigDecimal("0.5")).setScale(0, BigDecimal.ROUND_HALF_UP).intValue();
        log.info("快手参与有效率排名的拍摄中间50%人数为：" + shotTop30);
        //测试完删除
        // shotTop30 = 1;
        // shotMiddle50 = 1;

        //循环前30%有绩效的人数并计算其绩效，并入库
        for (int i = 0; i <= shotTop30 - 1; i++) {
            //总消耗--获取这个人所在项目的总消耗
            BigDecimal totalCost = userEfficientVideoMapMapper.getTotalCostByShotId(shotList.get(i).getUserId(), startTime, endTime);
            //总绩效=总消耗*提点比例（5‰）*提成比例（拍摄10%）
            BigDecimal totalPerformance = totalCost.multiply(new BigDecimal("0.005")).multiply(new BigDecimal("0.1"));

            Performance performance = new Performance();
            performance.setAppType(2); //1头条 2快手
            performance.setRoleId(shotList.get(i).getRoleId());
            performance.setQuarter(thisQuarter);
            performance.setYear(thisYear);
            performance.setCommissionRate(new BigDecimal("0.005"));
            performance.setRoleCommissionRate(new BigDecimal("0.1"));
            performance.setTotalCost(totalCost);
            performance.setTotalPerformance(totalPerformance);
            performance.setStartTime(startTime);
            performance.setEndTime(endTime);
            performance.setVideoEfficiency(shotList.get(i).getEffiRate().setScale(4, BigDecimal.ROUND_HALF_UP));
            performance.setUserId(shotList.get(i).getUserId());
            performanceMapper.insert(performance);
        }

        //去掉list中前30的数据
        shotList.subList(0, shotTop30 - 1).clear();

        //循环中间50%有绩效的人，计算绩效并入库
        for (int i = 0; i <= shotMiddle50 - 1; i++) {
            //总消耗--获取这个人所在项目的总消耗
            BigDecimal totalCost = userEfficientVideoMapMapper.getTotalCostByShotId(shotList.get(i).getUserId(), startTime, endTime);
            //总绩效=总消耗*提点比例（3‰）*提成比例（拍摄10%）
            BigDecimal totalPerformance = totalCost.multiply(new BigDecimal("0.003")).multiply(new BigDecimal("0.1"));

            Performance performance = new Performance();
            performance.setAppType(2); //1头条 2快手
            performance.setRoleId(shotList.get(i).getRoleId());
            performance.setQuarter(thisQuarter);
            performance.setYear(thisYear);
            performance.setCommissionRate(new BigDecimal("0.003"));
            performance.setRoleCommissionRate(new BigDecimal("0.1"));
            performance.setTotalCost(totalCost);
            performance.setTotalPerformance(totalPerformance);
            performance.setStartTime(startTime);
            performance.setEndTime(endTime);
            performance.setVideoEfficiency(shotList.get(i).getEffiRate().setScale(4, BigDecimal.ROUND_HALF_UP));
            performance.setUserId(shotList.get(i).getUserId());
            performanceMapper.insert(performance);
        }
        log.info(thisYear + "年第" + thisQuarter + "季度快手拍摄季度绩效计算开始");
    }
    /////////////////////////////////////////////////////////////////////////////////////////////快手拍摄绩效计算end

    //////////////////////////////////////////////////////////////////////////////////////////////////////////快手剪辑绩效计算start
    private void kuaishouClipPerformance(int thisYear, int thisQuarter, String startTime, String endTime) {
        log.info(thisYear + "年第" + thisQuarter + "季度快手剪辑季度绩效计算开始");
        ///根据角色（剪辑）查询出相对应角色所有的人
        //List<UserDto> clipList = sysRoleMapper.getUserByRoleCode("clip"); //剪辑
        List<UserDto2> clipList = userEfficientVideoMapMapper.getKuaishouClipList(); //剪辑
        for (UserDto2 userDto : clipList) {
            //有效视频数量
            int effiVideoCount = userEfficientVideoMapMapper.getEffiVideoCountByUserId(2, thisYear, thisQuarter, userDto.getUserId(), "f38d8d70cf7ec50d5357a749e4dbf8ee");
            //总视频数量
            int videoCount = userEfficientVideoMapMapper.getQuarterMaterialCountByUserId("clip", startTime, endTime, userDto.getUserId());
            //视频有效率
            BigDecimal effiRate = new BigDecimal(String.valueOf(effiVideoCount)).divide(new BigDecimal(String.valueOf(videoCount)));
            userDto.setEffiRate(effiRate);
        }
        Collections.sort(clipList, new EffiRateComparator());

        //获取剪辑人数并按照提点比例计算每个梯度的人数
        int clipLength = clipList.size();
        log.info("快手参与有效率排名的剪辑总人数为：" + clipLength);
        //获取前30%和中间50%的人数（视频有效率四舍五入）
        int clipTop30 = new BigDecimal(String.valueOf(clipLength)).multiply(new BigDecimal("0.3")).setScale(0, BigDecimal.ROUND_HALF_UP).intValue();
        log.info("快手参与有效率排名的剪辑前30%人数为：" + clipTop30);
        int clipMiddle50 = new BigDecimal(String.valueOf(clipLength)).multiply(new BigDecimal("0.5")).setScale(0, BigDecimal.ROUND_HALF_UP).intValue();
        log.info("快手参与有效率排名的剪辑中间50%人数为：" + clipMiddle50);
        //测试完删除
        //clipTop30 = 1;
        //clipMiddle50 = 1;

        //循环前30%有绩效的人数并计算其绩效，并入库
        for (int i = 0; i <= clipTop30 - 1; i++) {
            //总消耗--获取这个人所在项目的总消耗
            BigDecimal totalCost = userEfficientVideoMapMapper.getTotalCostByClipId(clipList.get(i).getUserId(), startTime, endTime);
            //总绩效=总消耗*提点比例（5‰）*提成比例（剪辑30%）
            BigDecimal totalPerformance = totalCost.multiply(new BigDecimal("0.005")).multiply(new BigDecimal("0.3"));

            Performance performance = new Performance();
            performance.setAppType(2); //1头条 2快手
            performance.setRoleId(clipList.get(i).getRoleId());
            performance.setQuarter(thisQuarter);
            performance.setYear(thisYear);
            performance.setCommissionRate(new BigDecimal("0.005"));
            performance.setRoleCommissionRate(new BigDecimal("0.3"));
            performance.setTotalCost(totalCost);
            performance.setTotalPerformance(totalPerformance);
            performance.setStartTime(startTime);
            performance.setEndTime(endTime);
            performance.setVideoEfficiency(clipList.get(i).getEffiRate().setScale(4, BigDecimal.ROUND_HALF_UP));
            performance.setUserId(clipList.get(i).getUserId());
            performanceMapper.insert(performance);
        }

        //去掉list中前30的数据
        clipList.subList(0, clipTop30 - 1).clear();

        //循环中间50%有绩效的人，计算绩效并入库
        for (int i = 0; i <= clipMiddle50 - 1; i++) {
            //总消耗--获取这个人所在项目的总消耗
            BigDecimal totalCost = userEfficientVideoMapMapper.getTotalCostByClipId(clipList.get(i).getUserId(), startTime, endTime);
            //总绩效=总消耗*提点比例（3‰）*提成比例（剪辑30%）
            BigDecimal totalPerformance = totalCost.multiply(new BigDecimal("0.003")).multiply(new BigDecimal("0.3"));

            Performance performance = new Performance();
            performance.setAppType(2); //1头条 2快手
            performance.setRoleId(clipList.get(i).getRoleId());
            performance.setQuarter(thisQuarter);
            performance.setYear(thisYear);
            performance.setCommissionRate(new BigDecimal("0.003"));
            performance.setRoleCommissionRate(new BigDecimal("0.3"));
            performance.setTotalCost(totalCost);
            performance.setTotalPerformance(totalPerformance);
            performance.setStartTime(startTime);
            performance.setEndTime(endTime);
            performance.setVideoEfficiency(clipList.get(i).getEffiRate().setScale(4, BigDecimal.ROUND_HALF_UP));
            performance.setUserId(clipList.get(i).getUserId());
            performanceMapper.insert(performance);
        }
        log.info(thisYear + "年第" + thisQuarter + "季度快手剪辑季度绩效计算开始");
    }
    //////////////////////////////////////////////////////////////////////////////////////////////////快手剪辑绩效计算end

    //////////////////////////////////////////////////////////////////////////////////////////////快手平面start
    //目前先按照同一个项目最后时间的所有平面设计师平分消耗绩效来计算
    public void kuaishouPlanePerformance(int thisYear, int thisQuarter, String startTime, String endTime) {
        log.info(thisYear + "年第" + thisQuarter + "季度快手平面设计师季度绩效计算开始");
        //搜索所有项目，循环并计算时间段内的项目总消耗，并查询其中有没有平面设计师
        //如果有并且不止一个就平分；创建一个list，将设计师的userDto加入进来，然后可以在不同的项目中累加（需要增加一张表格，然后记录其中的每一个项目消耗）
        Map<String, UserDto2> planeMap = new HashMap<>();

        //TODO ctop_project表有修改，需要进行相对应的变化
        //获取所有project根据媒体类型获取快手下的项目类型 1头条 2快手
        List<ProjectDTO> projectDTOS = userEfficientVideoMapMapper.getProjects("2");

        //循环project
        for (ProjectDTO projectDTO : projectDTOS) {
            //判断是否是北京的部门负责的项目，如果不是的话跳出
            String orgCode = userEfficientVideoMapMapper.getOrgCodeByProjectId(projectDTO.getProjectId());
            if (!orgCode.startsWith("A01")) {
                log.info("此项目不属于北京公司，跳出进入下一个项目计算");
                continue;
            }

            List<UserDto2> planeList = userEfficientVideoMapMapper.getPlaneByProjectId(projectDTO.getProjectId());
            //此项目当中平面数量,平面数量为0的时候说明此project当中没有平面设计师，故跳过
            int size = planeList.size();
            log.info(projectDTO.getProjectName() + "中有" + size + "个平面设计");
            if (size == 0) {
                continue;
            }
            BigDecimal quarterCost = userEfficientVideoMapMapper.getProjectQuarterCostById(startTime, endTime, projectDTO.getProjectId());
            BigDecimal averageQuarterCost = quarterCost.divide(new BigDecimal(String.valueOf(size)));

            //循环项目下所有平面设计
            for (UserDto2 plane : planeList) {
                //将金额和项目插入数据库当中（需要新建表）
                plane.setEffiRate(averageQuarterCost);
                //插入数据
                PlaneProjectCost planeProjectCost = new PlaneProjectCost();
                planeProjectCost.setAppType(2); //1头条 2快手
                planeProjectCost.setProjectId(projectDTO.getProjectId());
                planeProjectCost.setYear(thisYear);
                planeProjectCost.setQuarter(thisQuarter);
                planeProjectCost.setProjectTotalCost(quarterCost);
                planeProjectCost.setProjectCost(averageQuarterCost);
                planeProjectCost.setUserId(plane.getUserId());
                //增加平面人数、和项目单独的绩效
                planeProjectCost.setProjectPlaneCount(planeList.size());
                planeProjectCost.setProjectPerformance(averageQuarterCost.multiply(new BigDecimal("0.05")));
                if (null == planeMap.get(plane.getUserId())) {
                    plane.setEffiRate(averageQuarterCost);
                    planeMap.put(plane.getUserId(), plane);
                } else {
                    //将增加的金额增加到
                    planeMap.get(plane.getUserId()).setEffiRate(planeMap.get(plane.getUserId()).getEffiRate().add(plane.getEffiRate()));
                }
                planeProjectCostMapper.insert(planeProjectCost);
            }
        }

        //循环planeMap插入数据
        for (String key : planeMap.keySet()) {
            Performance performance = new Performance();
            performance.setAppType(2); //1头条 2快手
            performance.setRoleId(planeMap.get(key).getRoleId());
            performance.setQuarter(thisQuarter);
            performance.setYear(thisYear);
            //performance.setCommissionRate(new BigDecimal("0.05"));
            performance.setRoleCommissionRate(new BigDecimal("0.05"));
            performance.setTotalCost(planeMap.get(key).getEffiRate());
            performance.setTotalPerformance(planeMap.get(key).getEffiRate().multiply(new BigDecimal("0.05")));
            performance.setStartTime(startTime);
            performance.setEndTime(endTime);
            performance.setUserId(planeMap.get(key).getUserId());
            performanceMapper.insert(performance);
        }
        log.info(thisYear + "年第" + thisQuarter + "季度快手平面设计师季度绩效计算结束");
    }
    ///////////////////////////////////////////////////////////////////////////////////////////////////////快手平面end

    //////////////////////////////////////////////////////////////////////////////////////////////////快手设计师leader绩效计算start
    private void kuaishouDesignLeaderPerformance(int thisYear, int thisQuarter, String startTime, String endTime) {
        log.info(thisYear + "年第" + thisQuarter + "季度快手设计师leader季度绩效计算开始");
        //直接查询出来各种媒体类型的设计组长
        List<UserDto2> designLeaderList = userEfficientVideoMapMapper.getKuaishouDesignLeaderList();

        //循环list然后根据查出leader组下的剪辑userId
        for (UserDto2 userDto : designLeaderList) {
            List<String> clipList = userEfficientVideoMapMapper.getClip(userDto.getDepId());
            BigDecimal totalCost = BigDecimal.ZERO; //剪辑总消耗

            for (String clipUserId : clipList) {
                BigDecimal clipCost = userEfficientVideoMapMapper.getClipQuarterCost(2, thisQuarter, thisYear, clipUserId);  //媒体类型 //1头条 2快手
                totalCost = totalCost.add(clipCost);
            }

            //总绩效=剪辑消耗总合 * 平台比例 * 20%
            BigDecimal totalPerformance = totalCost.multiply(new BigDecimal("0.003")).multiply(new BigDecimal("0.2"));

            Performance performance = new Performance();
            performance.setAppType(2); //1头条 2快手
            performance.setRoleId(userDto.getRoleId());
            performance.setQuarter(thisQuarter);
            performance.setYear(thisYear);
            performance.setCommissionRate(new BigDecimal("0.003"));
            performance.setTotalCost(totalCost);
            performance.setTotalPerformance(totalPerformance);
            performance.setStartTime(startTime);
            performance.setEndTime(endTime);
            performance.setRoleCommissionRate(new BigDecimal("0.2"));
            performance.setUserId(userDto.getUserId());
            performanceMapper.insert(performance);
        }
        log.info(thisYear + "年第" + thisQuarter + "季度快手设计师leader季度绩效计算结束");
    }
    //////////////////////////////////////////////////////////////////////////////////////////////////快手设计师leader绩效计算end

    ////////////////////////////////////////////////////////////////////////////////////////////
    //头条
    ////////////////////////////////////////////////////////////////////////////////////////////
    //头条编导季度绩效计算 start
    private void toutiaoPlanPerformance(int thisYear, int thisQuarter, String startTime, String endTime) {
        log.info(thisYear + "年第" + thisQuarter + "季度头条编导季度绩效计算开始");
        //1.根据角色（编导）查询出相对应角色所有的人
        List<UserDto2> planList = userEfficientVideoMapMapper.getToutiaoPlanList(); //编导
        //循环编导列表计算视频有效率并排序
        for (UserDto2 userDto : planList) {
            //有效视频数量
            int effiVideoCount = userEfficientVideoMapMapper.getEffiVideoCountByUserId(1, thisYear, thisQuarter, userDto.getUserId(), "0214283aa16f943efbb149ea4bb18f18");  //媒体类型 //1头条 2快手
            //总视频数量
            int videoCount = userEfficientVideoMapMapper.getQuarterMaterialCountByUserId("plan", startTime, endTime, userDto.getUserId());
            //视频有效率
            BigDecimal effiRate = new BigDecimal(String.valueOf(effiVideoCount)).divide(new BigDecimal(String.valueOf(videoCount)));
            userDto.setEffiRate(effiRate);
        }
        Collections.sort(planList, new EffiRateComparator());

        //获取编导人数并按照提点比例计算每个梯度的人数
        int planLength = planList.size();
        log.info("头条参与有效率排名的编导总人数为：" + planLength);
        //获取前30%和中间50%的人数（视频有效率四舍五入）
        int top30 = new BigDecimal(String.valueOf(planLength)).multiply(new BigDecimal("0.3")).setScale(0, BigDecimal.ROUND_HALF_UP).intValue();
        log.info("头条参与有效率排名的编导前30%人数为：" + top30);
        int middle50 = new BigDecimal(String.valueOf(planLength)).multiply(new BigDecimal("0.5")).setScale(0, BigDecimal.ROUND_HALF_UP).intValue();
        log.info("头条参与有效率排名的编导中间50%人数为：" + middle50);

        //循环前30%有绩效的人数并计算其绩效，并入库
        for (int i = 0; i <= top30 - 1; i++) {
            //头条编导总消耗
            BigDecimal totalCost = userEfficientVideoMapMapper.toutiaoGetTotalCostByPlanId(planList.get(i).getUserId(), startTime, endTime);
            //总绩效=总消耗*提点比例（5‰）*提成比例（编导40%）
            BigDecimal totalPerformance = totalCost.multiply(new BigDecimal("0.003")).multiply(new BigDecimal("0.4"));

            Performance performance = new Performance();
            performance.setAppType(1); //1头条 2快手
            performance.setRoleId(planList.get(i).getRoleId());
            performance.setQuarter(thisQuarter);
            performance.setYear(thisYear);
            performance.setCommissionRate(new BigDecimal("0.003"));
            performance.setRoleCommissionRate(new BigDecimal("0.4"));
            performance.setTotalCost(totalCost);
            performance.setTotalPerformance(totalPerformance);
            performance.setStartTime(startTime);
            performance.setEndTime(endTime);
            performance.setVideoEfficiency(planList.get(i).getEffiRate().setScale(4, BigDecimal.ROUND_HALF_UP));
            performance.setUserId(planList.get(i).getUserId());
            performanceMapper.insert(performance);
        }

        //去掉list中前30的数据
        planList.subList(0, top30 - 1).clear();

        //循环中间50%有绩效的人，计算绩效并入库
        for (int i = 0; i <= middle50 - 1; i++) {
            //这个编导的总消耗
            BigDecimal totalCost = userEfficientVideoMapMapper.toutiaoGetTotalCostByPlanId(planList.get(i).getUserId(), startTime, endTime);
            //总绩效=总消耗*提点比例（2‰）*提成比例（编导40%）
            BigDecimal totalPerformance = totalCost.multiply(new BigDecimal("0.002")).multiply(new BigDecimal("0.4"));

            Performance performance = new Performance();
            performance.setAppType(1); //1头条 2快手
            performance.setRoleId(planList.get(i).getRoleId());
            performance.setQuarter(thisQuarter);
            performance.setYear(thisYear);
            performance.setCommissionRate(new BigDecimal("0.002"));
            performance.setRoleCommissionRate(new BigDecimal("0.4"));
            performance.setTotalCost(totalCost);
            performance.setTotalPerformance(totalPerformance);
            performance.setStartTime(startTime);
            performance.setEndTime(endTime);
            performance.setVideoEfficiency(planList.get(i).getEffiRate().setScale(4, BigDecimal.ROUND_HALF_UP));
            performance.setUserId(planList.get(i).getUserId());
            performanceMapper.insert(performance);
        }
        log.info(thisYear + "年第" + thisQuarter + "季度头条编导季度绩效计算结束");
    }
    //////////////////////////////////////////////////////////////////////////////////////////////////////头条编导end

    /////////////////////////////////////////////////////////////////////////////////////////////////////头条拍摄绩效start
    private void toutiaoShotPerformance(int thisYear, int thisQuarter, String startTime, String endTime) {
        log.info(thisYear + "年第" + thisQuarter + "季度头条拍摄季度绩效计算开始");
        //根据角色（拍摄）查询出相对应角色所有的人
        //List<UserDto> shotList = sysRoleMapper.getUserByRoleCode("shot"); //拍摄
        List<UserDto2> shotList = userEfficientVideoMapMapper.getToutiaoShotList(); //拍摄
        for (UserDto2 userDto : shotList) {
            //有效视频数量
            int effiVideoCount = userEfficientVideoMapMapper.getEffiVideoCountByUserId(1, thisYear, thisQuarter, userDto.getUserId(), "7bff9afed625aeeabca6bffe3c189183");
            //总视频数量
            int videoCount = userEfficientVideoMapMapper.getQuarterMaterialCountByUserId("shot", startTime, endTime, userDto.getUserId());
            //视频有效率
            BigDecimal effiRate = new BigDecimal(String.valueOf(effiVideoCount)).divide(new BigDecimal(String.valueOf(videoCount)));
            userDto.setEffiRate(effiRate);
        }
        Collections.sort(shotList, new EffiRateComparator());

        //获取编导人数并按照提点比例计算每个梯度的人数
        int shotLength = shotList.size();
        log.info("头条参与有效率排名的拍摄总人数为：" + shotLength);
        //获取前30%和中间50%的人数（视频有效率四舍五入）
        int shotTop30 = new BigDecimal(String.valueOf(shotLength)).multiply(new BigDecimal("0.3")).setScale(0, BigDecimal.ROUND_HALF_UP).intValue();
        log.info("头条参与有效率排名的拍摄前30%人数为：" + shotTop30);
        int shotMiddle50 = new BigDecimal(String.valueOf(shotLength)).multiply(new BigDecimal("0.5")).setScale(0, BigDecimal.ROUND_HALF_UP).intValue();
        log.info("头条参与有效率排名的拍摄中间50%人数为：" + shotMiddle50);

        //循环前30%有绩效的人数并计算其绩效，并入库
        for (int i = 0; i <= shotTop30 - 1; i++) {
            //总消耗--获取这个人所在项目的总消耗
            BigDecimal totalCost = userEfficientVideoMapMapper.toutiaoGetTotalCostByShotId(shotList.get(i).getUserId(), startTime, endTime);
            //总绩效=总消耗*提点比例（5‰）*提成比例（拍摄10%）
            BigDecimal totalPerformance = totalCost.multiply(new BigDecimal("0.003")).multiply(new BigDecimal("0.1"));

            Performance performance = new Performance();
            performance.setAppType(1); //1头条 2快手
            performance.setRoleId(shotList.get(i).getRoleId());
            performance.setQuarter(thisQuarter);
            performance.setYear(thisYear);
            performance.setCommissionRate(new BigDecimal("0.003"));
            performance.setRoleCommissionRate(new BigDecimal("0.1"));
            performance.setTotalCost(totalCost);
            performance.setTotalPerformance(totalPerformance);
            performance.setStartTime(startTime);
            performance.setEndTime(endTime);
            performance.setVideoEfficiency(shotList.get(i).getEffiRate().setScale(4, BigDecimal.ROUND_HALF_UP));
            performance.setUserId(shotList.get(i).getUserId());
            performanceMapper.insert(performance);
        }

        //去掉list中前30的数据
        shotList.subList(0, shotTop30 - 1).clear();

        //循环中间50%有绩效的人，计算绩效并入库
        for (int i = 0; i <= shotMiddle50 - 1; i++) {
            //总消耗--获取这个人所在项目的总消耗
            BigDecimal totalCost = userEfficientVideoMapMapper.toutiaoGetTotalCostByShotId(shotList.get(i).getUserId(), startTime, endTime);
            //总绩效=总消耗*提点比例（3‰）*提成比例（拍摄10%）
            BigDecimal totalPerformance = totalCost.multiply(new BigDecimal("0.002")).multiply(new BigDecimal("0.1"));

            Performance performance = new Performance();
            performance.setAppType(1); //1头条 2快手
            performance.setRoleId(shotList.get(i).getRoleId());
            performance.setQuarter(thisQuarter);
            performance.setYear(thisYear);
            performance.setCommissionRate(new BigDecimal("0.002"));
            performance.setRoleCommissionRate(new BigDecimal("0.1"));
            performance.setTotalCost(totalCost);
            performance.setTotalPerformance(totalPerformance);
            performance.setStartTime(startTime);
            performance.setEndTime(endTime);
            performance.setVideoEfficiency(shotList.get(i).getEffiRate().setScale(4, BigDecimal.ROUND_HALF_UP));
            performance.setUserId(shotList.get(i).getUserId());
            performanceMapper.insert(performance);
        }
        log.info(thisYear + "年第" + thisQuarter + "季度头条拍摄季度绩效计算结束");
    }
    ///////////////////////////////////////////////////////////////////////////头条拍摄绩效end

    ////////////////////////////////////////////////////////////////////////////头条剪辑绩效计算start
    private void toutiaoClipPerformance(int thisYear, int thisQuarter, String startTime, String endTime) {
        log.info(thisYear + "年第" + thisQuarter + "季度头条剪辑季度绩效计算开始");
        ///根据角色（剪辑）查询出相对应角色所有的人
        //List<UserDto> clipList = sysRoleMapper.getUserByRoleCode("clip"); //剪辑
        List<UserDto2> clipList = userEfficientVideoMapMapper.getKuaishouClipList(); //剪辑
        for (UserDto2 userDto : clipList) {
            //有效视频数量 //1头条 2快手
            int effiVideoCount = userEfficientVideoMapMapper.getEffiVideoCountByUserId(1, thisYear, thisQuarter, userDto.getUserId(), "f38d8d70cf7ec50d5357a749e4dbf8ee");
            //总视频数量
            int videoCount = userEfficientVideoMapMapper.getQuarterMaterialCountByUserId("clip", startTime, endTime, userDto.getUserId());
            //视频有效率
            BigDecimal effiRate = new BigDecimal(String.valueOf(effiVideoCount)).divide(new BigDecimal(String.valueOf(videoCount)));
            userDto.setEffiRate(effiRate);
        }
        Collections.sort(clipList, new EffiRateComparator());

        //获取剪辑人数并按照提点比例计算每个梯度的人数
        int clipLength = clipList.size();
        log.info("头条参与有效率排名的剪辑总人数为：" + clipLength);
        //获取前30%和中间50%的人数（视频有效率四舍五入）
        int clipTop30 = new BigDecimal(String.valueOf(clipLength)).multiply(new BigDecimal("0.3")).setScale(0, BigDecimal.ROUND_HALF_UP).intValue();
        log.info("头条参与有效率排名的前30%的人数为：" + clipTop30);
        int clipMiddle50 = new BigDecimal(String.valueOf(clipLength)).multiply(new BigDecimal("0.5")).setScale(0, BigDecimal.ROUND_HALF_UP).intValue();
        log.info("头条参与有效率排名的中间50%的人数为：" + clipMiddle50);

        //循环前30%有绩效的人数并计算其绩效，并入库
        for (int i = 0; i <= clipTop30 - 1; i++) {
            //总消耗--获取这个人所在项目的总消耗
            BigDecimal totalCost = userEfficientVideoMapMapper.toutiaoGetTotalCostByClipId(clipList.get(i).getUserId(), startTime, endTime);
            //总绩效=总消耗*提点比例（5‰）*提成比例（剪辑30%）
            BigDecimal totalPerformance = totalCost.multiply(new BigDecimal("0.003")).multiply(new BigDecimal("0.3"));

            Performance performance = new Performance();
            performance.setAppType(1); //1头条 2快手
            performance.setRoleId(clipList.get(i).getRoleId());
            performance.setQuarter(thisQuarter);
            performance.setYear(thisYear);
            performance.setCommissionRate(new BigDecimal("0.003"));
            performance.setRoleCommissionRate(new BigDecimal("0.3"));
            performance.setTotalCost(totalCost);
            performance.setTotalPerformance(totalPerformance);
            performance.setStartTime(startTime);
            performance.setEndTime(endTime);
            performance.setVideoEfficiency(clipList.get(i).getEffiRate().setScale(4, BigDecimal.ROUND_HALF_UP));
            performance.setUserId(clipList.get(i).getUserId());
            performanceMapper.insert(performance);
        }

        //去掉list中前30的数据
        clipList.subList(0, clipTop30 - 1).clear();

        //循环中间50%有绩效的人，计算绩效并入库
        for (int i = 0; i <= clipMiddle50 - 1; i++) {
            //总消耗--获取这个人所在项目的总消耗
            BigDecimal totalCost = userEfficientVideoMapMapper.toutiaoGetTotalCostByClipId(clipList.get(i).getUserId(), startTime, endTime);
            //总绩效=总消耗*提点比例（3‰）*提成比例（剪辑30%）
            BigDecimal totalPerformance = totalCost.multiply(new BigDecimal("0.002")).multiply(new BigDecimal("0.3"));

            Performance performance = new Performance();
            performance.setAppType(1); //1头条 2快手
            performance.setRoleId(clipList.get(i).getRoleId());
            performance.setQuarter(thisQuarter);
            performance.setYear(thisYear);
            performance.setCommissionRate(new BigDecimal("0.002"));
            performance.setRoleCommissionRate(new BigDecimal("0.3"));
            performance.setTotalCost(totalCost);
            performance.setTotalPerformance(totalPerformance);
            performance.setStartTime(startTime);
            performance.setEndTime(endTime);
            performance.setVideoEfficiency(clipList.get(i).getEffiRate().setScale(4, BigDecimal.ROUND_HALF_UP));
            performance.setUserId(clipList.get(i).getUserId());
            performanceMapper.insert(performance);
        }
        log.info(thisYear + "年第" + thisQuarter + "季度头条剪辑季度绩效计算结束");
    }
    ////////////////////////////////////////////////////////////////////////////////////头条剪辑绩效计算end

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////头条平面设计师start
    //目前先按照同一个项目最后时间的所有平面设计师平分消耗绩效来计算
    public void toutiaoPlanePerformance(int thisYear, int thisQuarter, String startTime, String endTime) {
        log.info(thisYear + "年第" + thisQuarter + "季度头条平面季度绩效计算开始");
        //搜索所有项目，循环并计算时间段内的项目总消耗，并查询其中有没有平面设计师
        //如果有并且不止一个就平分；创建一个list，将设计师的userDto加入进来，然后可以在不同的项目中累加（需要增加一张表格，然后记录其中的每一个项目消耗）
        Map<String, UserDto2> planeMap = new HashMap<>();

        //TODO ctop_project表有修改，需要进行相对应的变化
        //获取所有project根据媒体类型获取快手下的项目类型 1-头条 2-快手
        List<ProjectDTO> projectDTOS = userEfficientVideoMapMapper.getProjects("1");
        System.out.println("头条项目数量为：" + projectDTOS.size());

        //循环project
        for (ProjectDTO projectDTO : projectDTOS) {
            //判断是否是北京的部门负责的项目，如果不是的话跳出
            String orgCode = userEfficientVideoMapMapper.getOrgCodeByProjectId(projectDTO.getProjectId());
            if (!orgCode.startsWith("A01")) {
                log.info(projectDTO.getProjectName() + "不是北京的部门的项目，跳出");
                continue;
            }

            List<UserDto2> planeList = userEfficientVideoMapMapper.getPlaneByProjectId(projectDTO.getProjectId());
            //此项目当中平面数量,平面数量为0的时候说明此project当中没有平面设计师，故跳过
            int size = planeList.size();
            if (size == 0) {
                continue;
            }
            BigDecimal quarterCost = userEfficientVideoMapMapper.getProjectQuarterCostById(startTime, endTime, projectDTO.getProjectId());
            BigDecimal averageQuarterCost = quarterCost.divide(new BigDecimal(String.valueOf(size)));

            //循环项目下所有平面设计
            for (UserDto2 plane : planeList) {
                //将金额和项目插入数据库当中（需要新建表）
                plane.setEffiRate(averageQuarterCost);
                //插入数据
                PlaneProjectCost planeProjectCost = new PlaneProjectCost();
                planeProjectCost.setAppType(1); //1头条 2快手
                planeProjectCost.setProjectId(projectDTO.getProjectId());
                planeProjectCost.setYear(thisYear);
                planeProjectCost.setQuarter(thisQuarter);
                planeProjectCost.setProjectTotalCost(quarterCost);
                planeProjectCost.setProjectCost(averageQuarterCost);
                planeProjectCost.setUserId(plane.getUserId());
                //增加平面人数、和项目单独的绩效
                planeProjectCost.setProjectPlaneCount(planeList.size());
                planeProjectCost.setProjectPerformance(averageQuarterCost.multiply(new BigDecimal("0.05")));

                if (null == planeMap.get(plane.getUserId())) {
                    plane.setEffiRate(averageQuarterCost);
                    planeMap.put("plane.getUserId()", plane);
                } else {
                    //将增加的金额增加到
                    planeMap.get(plane.getUserId()).setEffiRate(planeMap.get(plane.getUserId()).getEffiRate().add(plane.getEffiRate()));
                }
                planeProjectCostMapper.insert(planeProjectCost);
            }
        }

        //循环planeMap插入数据
        for (String key : planeMap.keySet()) {
            Performance performance = new Performance();
            performance.setAppType(1); //1头条 2快手
            performance.setRoleId(planeMap.get(key).getRoleId());
            performance.setQuarter(thisQuarter);
            performance.setYear(thisYear);
            //performance.setCommissionRate(new BigDecimal("0.05"));
            performance.setTotalCost(planeMap.get(key).getEffiRate());
            performance.setTotalPerformance(planeMap.get(key).getEffiRate().multiply(new BigDecimal("0.05")));
            performance.setRoleCommissionRate(new BigDecimal("0.05"));
            performance.setStartTime(startTime);
            performance.setEndTime(endTime);
            performance.setUserId(planeMap.get(key).getUserId());
            performanceMapper.insert(performance);
        }
        log.info(thisYear + "年第" + thisQuarter + "季度头条平面季度绩效计算结束");
    }
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////头条平面设计师绩效end

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////头条设计师leader绩效 start
    private void toutiaoDesignLeaderPerformance(int thisYear, int thisQuarter, String startTime, String endTime) {
        log.info(thisYear + "年第" + thisQuarter + "季度头条设计师leader季度绩效计算开始");
        //直接查询出来各种媒体类型的设计组长
        List<UserDto2> designLeaderList = userEfficientVideoMapMapper.getKuaishouDesignLeaderList();

        //循环list然后根据查出leader组下的剪辑userId
        for (UserDto2 userDto : designLeaderList) {
            List<String> clipList = userEfficientVideoMapMapper.getClip(userDto.getDepId());
            BigDecimal totalCost = BigDecimal.ZERO; //剪辑总消耗

            for (String clipUserId : clipList) {
                BigDecimal clipCost = userEfficientVideoMapMapper.getClipQuarterCost(1, thisQuarter, thisYear, clipUserId);  //媒体类型 //1头条 2快手
                totalCost = totalCost.add(clipCost);
            }

            //总绩效=小组下所有剪辑消耗总合 * 平台比例 * 20%
            BigDecimal totalPerformance = totalCost.multiply(new BigDecimal("0.002")).multiply(new BigDecimal("0.2"));

            Performance performance = new Performance();
            performance.setAppType(1); //1头条 2快手
            performance.setRoleId(userDto.getRoleId());
            performance.setQuarter(thisQuarter);
            performance.setYear(thisYear);
            performance.setCommissionRate(new BigDecimal("0.002")); //千分之
            performance.setRoleCommissionRate(new BigDecimal("0.2"));
            performance.setTotalCost(totalCost);
            performance.setTotalPerformance(totalPerformance);
            performance.setStartTime(startTime);
            performance.setEndTime(endTime);
            performance.setUserId(userDto.getUserId());
            performanceMapper.insert(performance);
        }
        log.info(thisYear + "年第" + thisQuarter + "季度头条设计师leader季度绩效计算开始");
    }

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////头条设计师leader绩效 end
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //绩效计算结束
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //页面接口 start
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //1
    //上个季度个人的有效数量、素材总数、素材有效率、总跑量
    //{"effiVideoCount有效数":70,"videoCount素材总数":200,"effiRate":40,"cost":2019;performance绩效":20000}
    @Override
    public Map<String, Object> getPerformanceInfo() {
        Map<String, Object> performanceInfoMap = new HashMap<>();
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        //String roleCode = sysRoleMapper.getRoleCodeByUserId(sysUser.getId());
        //LoginUser sysUser = new LoginUser();
        //sysUser.setId("9712328fddd1443faf566ddb87693658");
        String orgCode = sysUser.getOrgCode();
        Integer appType = null;
        if (null != orgCode && orgCode.startsWith("A01A08")) {//头条
            appType = 1;
        } else if (null != orgCode && orgCode.startsWith("A01A07")) {//快手
            appType = 2;
        }else{
            log.info("getPerformanceInfo接口" + sysUser.getRealname() + "orgCode不正确");
            return performanceInfoMap;
        }

        Map<String,Integer> quarterMap = DateUtils.lastQuarter(new Date());
        Integer year = quarterMap.get("year");
        Integer quarter = quarterMap.get("quarter");
        Map<String,String> startEndDateMap = DateUtils.quarterStartEndDate(year,quarter);
        String startDate = startEndDateMap.get("startDate");
        String endDate = startEndDateMap.get("endDate");

        //获取本人的视频总有效数量
        int effiVideoCount = userEfficientVideoMapMapper.getEffiVideoCountByUserId(appType, year, quarter, sysUser.getId());
        //获取本人的视频总数
        int videoCount = userEfficientVideoMapMapper.getQuarterMaterialCountByUserId(roleCode, startDate, endDate, sysUser.getId());
        //获取本人目前的视频有效率
        BigDecimal videoEffiRate = BigDecimal.ZERO;
        if (effiVideoCount != 0 && videoCount != 0) {
            videoEffiRate = new BigDecimal(String.valueOf(effiVideoCount)).divide(new BigDecimal(String.valueOf(videoCount))).multiply(new BigDecimal("100")).setScale(2, BigDecimal.ROUND_HALF_UP);
        }

        BigDecimal cost = userEfficientVideoMapMapper.getQuarterCost(sysUser.getId(), year, quarter, appType);
        BigDecimal performance = userEfficientVideoMapMapper.getQuarterPerformance(sysUser.getId(), year, quarter, appType);

        performanceInfoMap.put("effiVideoCount", effiVideoCount);
        performanceInfoMap.put("videoCount", videoCount);
        performanceInfoMap.put("effiRate", videoEffiRate);
        performanceInfoMap.put("cost", cost);
        performanceInfoMap.put("performance",performance);

        log.info(roleCode + ":" + sysUser.getRealname() + "getPerformanceInfo接口结束");
        return performanceInfoMap;

        ////////////////////////////////////////////////////////
        //获取前一天的时间
        *//*Date date = new Date();
        Date getDate = DateUtils.addDay(date, -1);
        String startDate = DateUtils.getQuarterStartDate(getDate);
        String endDate = DateUtils.getQuarterEndDate(getDate);
        String endNowDate = DateUtils.formatDate(getDate);

        String roleCode = sysRoleMapper.getRoleCodeByUserId(sysUser.getId());

        log.info(roleCode + ":" + sysUser.getRealname() + "getPerformanceInfo接口开始");

        Map<String, String> lastDateMap = getLastDateInfo();
        //获取本人的视频总有效数量
        int effiVideoCount = userEfficientVideoMapMapper.getEffiVideoCountByUserId(appType, Integer.parseInt(lastDateMap.get("year")), Integer.parseInt(lastDateMap.get("quarter")), sysUser.getId());
        //获取本人的视频总数
        int videoCount = userEfficientVideoMapMapper.getQuarterMaterialCountByUserId(roleCode, startDate, endNowDate, sysUser.getId());
        //获取本人目前的视频有效率
        BigDecimal videoEffiRate = BigDecimal.ZERO;
        if (effiVideoCount != 0 && videoCount != 0) {
            videoEffiRate = new BigDecimal(String.valueOf(effiVideoCount)).divide(new BigDecimal(String.valueOf(videoCount))).multiply(new BigDecimal("100")).setScale(2, BigDecimal.ROUND_HALF_UP);
        }
        //获取本人的总跑量
        BigDecimal cost = BigDecimal.ZERO;


        if (roleCode.equals("plan")) {
            cost = userEfficientVideoMapMapper.getTotalCostByPlanId(sysUser.getId(), startDate, endDate);
        } else if (roleCode.equals("shot")) {
            cost = userEfficientVideoMapMapper.getTotalCostByShotId(sysUser.getId(), startDate, endDate);
        } else if (roleCode.equals("clip")) {
            cost = userEfficientVideoMapMapper.getTotalCostByClipId(sysUser.getId(), startDate, endDate);
        }*//*
////////////////////////////////////////////////////////
    }

    //2
    //有效率排行榜
    //{"list":[{"name":"pipi","effiRate":50},{"name":"pipixia","effiRate":40},{"name":"pipiqiu","effiRate":30}],"performance":performance}
    @Override
    public Map<String, Object> effiRateRank(Integer size) {
        Map<String, Object> resultMap = new HashMap<>();
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        String orgCode = sysUser.getOrgCode();
        Integer appType = null;
        if (null != orgCode && orgCode.startsWith("A01A08")) {//头条
            appType = 1;
        } else if (null != orgCode && orgCode.startsWith("A01A07")) {//快手
            appType = 2;
        }

        Map<String,Integer> quarterMap = DateUtils.lastQuarter(new Date());
        Integer year = quarterMap.get("year");
        Integer quarter = quarterMap.get("quarter");
        Map<String,String> startEndDateMap = DateUtils.quarterStartEndDate(year,quarter);
        String startDate = startEndDateMap.get("startDate");
        String endDate = startEndDateMap.get("endDate");

        Map<String, String> lastDateMap = getLastDateInfo();
        String roleCode = sysRoleMapper.getRoleCodeByUserId(sysUser.getId());
        log.info(roleCode + ":" + sysUser.getRealname() + "有效率排行榜接口开始");

        List<UserDto2> userRoleList = new ArrayList<>();

        if(appType == 1){
            if (roleCode.equals("plan")) {
                userRoleList = userEfficientVideoMapMapper.getToutiaoPlanList(); //编导
            } else if (roleCode.equals("shot")) {
                userRoleList = userEfficientVideoMapMapper.getToutiaoShotList(); //拍摄
            } else if (roleCode.equals("clip")) {
                userRoleList = userEfficientVideoMapMapper.getToutiaoClipList(); //剪辑
            }
        }else{
            if (roleCode.equals("plan")) {
                userRoleList = userEfficientVideoMapMapper.getKuaishouPlanList(); //编导
            } else if (roleCode.equals("shot")) {
                userRoleList = userEfficientVideoMapMapper.getKuaishouShotList(); //拍摄
            } else if (roleCode.equals("clip")) {
                userRoleList = userEfficientVideoMapMapper.getKuaishouClipList(); //剪辑
            }
        }

        if (userRoleList != null) {
            for (UserDto2 userDto : userRoleList) {
                //有效视频数量
                int effiVideoCount = userEfficientVideoMapMapper.getEffiVideoCountByUserId(appType, Integer.parseInt(lastDateMap.get("year")), Integer.parseInt(lastDateMap.get("quarter")), userDto.getUserId());
                //总视频数量
                int videoCount = userEfficientVideoMapMapper.getQuarterMaterialCountByUserId(roleCode, startDate, endDate, userDto.getUserId());
                //视频有效率
                BigDecimal effiRate = BigDecimal.ZERO;
                if (effiVideoCount != 0 && videoCount != 0) {
                    effiRate = new BigDecimal(String.valueOf(effiVideoCount)).divide(new BigDecimal(String.valueOf(videoCount))).multiply(new BigDecimal("100")).setScale(2, BigDecimal.ROUND_HALF_UP);
                }
                userDto.setEffiRate(effiRate);
            }
            Collections.sort(userRoleList, new EffiRateComparator());

            //去掉list中前30的数据
            if (size != null) {
                userRoleList.subList(size, userRoleList.size()).clear();
            }

            resultMap.put("userRoleList", userRoleList);
        } else {
            resultMap.put("userRoleList", null);
        }

        log.info(roleCode + ":" + sysUser.getRealname() + "有效率排行榜接口结束");
        return resultMap;
    }

    //3
    //素材、素材跑量、封面点击数、封面曝光数、行为数
    //{"videoUrl":"http://dadfadfadf.mp4","videoCost":34.56,"pageClickCount":1200,"pageExposureCount":3939,"behaviorCount":5000}
    @Override
    public List<DesignMaterialInfo> materialInfoList() {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        //LoginUser sysUser = new LoginUser();
        //sysUser.setId("9712328fddd1443faf566ddb87693658");
        String roleCode = sysRoleMapper.getRoleCodeByUserId(sysUser.getId());
        log.info(roleCode + ":" + sysUser.getRealname() + "materialInfoList接口开始");

        String orgCode = sysUser.getOrgCode();
        Integer appType = null;
        if (null != orgCode && orgCode.startsWith("A01A08")) {//头条
            appType = 1;
        } else if (null != orgCode && orgCode.startsWith("A01A07")) {//快手
            appType = 2;
        }

        Map<String,Integer> quarterMap = DateUtils.lastQuarter(new Date());
        Integer year = quarterMap.get("year");
        Integer quarter = quarterMap.get("quarter");
        Map<String,String> startEndDateMap = DateUtils.quarterStartEndDate(year,quarter);
        String startDate = startEndDateMap.get("startDate");
        String endDate = startEndDateMap.get("endDate");

       *//* Date date = new Date();
        Date getDate = DateUtils.addDay(date, -1);
        String startDate = DateUtils.getQuarterStartDate(getDate);
        String endDate = DateUtils.getQuarterEndDate(getDate);
        String endNowDate = DateUtils.formatDate(getDate);*//*

        List<DesignMaterialInfo> materialInfoList = new ArrayList<>();

        if(appType == 1){
            materialInfoList = null;
        }else{
            materialInfoList = userEfficientVideoMapMapper.designMaterialInfoList(roleCode, sysUser.getId(), startDate, endDate);
        }

        log.info(roleCode + ":" + sysUser.getRealname() + "materialInfoList接口结束");
        return materialInfoList;
    }

    //4
    //点击+符号 投放账户、投放账户、跑量
    //{"videoAccount":"aabbcc","servingAccount":"ttyyhh","cost":34234.43}
    @Override
    public List<DesignMaterialInfo> materialAccountInfo(String materialInfoId) {
        log.info("materialAccountInfo接口开始");
        Date date = new Date();
        Date getDate = DateUtils.addDay(date, -1);
        String startDate = DateUtils.getQuarterStartDate(getDate);
        String endDate = DateUtils.getQuarterEndDate(getDate);
        String endNowDate = DateUtils.formatDate(getDate);

        List<DesignMaterialInfo> materialInfoList = userEfficientVideoMapMapper.getMaterialAccountInfoByMaterialId(materialInfoId, startDate, endNowDate);
        log.info("materialAccountInfo接口结束");
        return materialInfoList;
    }*/
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //页面接口 start
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //1
    //季度到前一天为止个人的有效数量、素材总数、素材有效率、总跑量
    //{"effiVideoCount有效数":70,"videoCount素材总数":200,"effiRate":40,"cost":2019;performance绩效":20000}
    public Map<String, Object> getPerformanceInfo() {
        Map<String, Object> performanceInfoMap = new HashMap<>();
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        //LoginUser sysUser = new LoginUser();
        //sysUser.setId("9712328fddd1443faf566ddb87693658");

        //获取前一天的时间
        Date date = new Date();
        Date getDate = DateUtils.addDay(date, -1);
        String startDate = DateUtils.getQuarterStartDate(getDate);
        String endDate = DateUtils.getQuarterEndDate(getDate);
        String endNowDate = DateUtils.formatDate(getDate);

        String roleCode = sysRoleMapper.getRoleCodeByUserId(sysUser.getId());

        log.info(roleCode + ":" + sysUser.getRealname() + "getPerformanceInfo接口开始");

        Map<String, String> lastDateMap = getLastDateInfo();
        //获取本人的视频总有效数量
        int effiVideoCount = userEfficientVideoMapMapper.getEffiVideoCountByUserId(2, Integer.parseInt(lastDateMap.get("year")), Integer.parseInt(lastDateMap.get("quarter")), sysUser.getId(), null);
        //获取本人的视频总数
        int videoCount = userEfficientVideoMapMapper.getQuarterMaterialCountByUserId(roleCode, startDate, endNowDate, sysUser.getId());
        //获取本人目前的视频有效率
        BigDecimal videoEffiRate = BigDecimal.ZERO;
        if (effiVideoCount != 0 && videoCount != 0) {
            videoEffiRate = new BigDecimal(String.valueOf(effiVideoCount)).divide(new BigDecimal(String.valueOf(videoCount))).multiply(new BigDecimal("100")).setScale(2, BigDecimal.ROUND_HALF_UP);
        }
        //获取本人的总跑量
        BigDecimal cost = BigDecimal.ZERO;

        if (roleCode.equals("plan")) {
            cost = userEfficientVideoMapMapper.getTotalCostByPlanId(sysUser.getId(), startDate, endDate);
        } else if (roleCode.equals("shot")) {
            cost = userEfficientVideoMapMapper.getTotalCostByShotId(sysUser.getId(), startDate, endDate);
        } else if (roleCode.equals("clip")) {
            cost = userEfficientVideoMapMapper.getTotalCostByClipId(sysUser.getId(), startDate, endDate);
        }

        performanceInfoMap.put("effiVideoCount", effiVideoCount);
        performanceInfoMap.put("videoCount", videoCount);
        performanceInfoMap.put("effiRate", videoEffiRate);
        performanceInfoMap.put("cost", cost);

        log.info(roleCode + ":" + sysUser.getRealname() + "getPerformanceInfo接口开始");
        return performanceInfoMap;
    }

    //2
    //有效率排行榜
    //{"list":[{"name":"pipi","effiRate":50},{"name":"pipixia","effiRate":40},{"name":"pipiqiu","effiRate":30}],"performance":performance}
    public Map<String, Object> effiRateRank(Integer size) {
        Map<String, Object> resultMap = new HashMap<>();
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        //LoginUser sysUser = new LoginUser();
        //sysUser.setId("9712328fddd1443faf566ddb87693658");

        //获取前一天的时间
        Date date = new Date();
        Date getDate = DateUtils.addDay(date, -1);
        String startDate = DateUtils.getQuarterStartDate(getDate);
        String endDate = DateUtils.getQuarterEndDate(getDate);
        String endNowDate = DateUtils.formatDate(getDate);
        Map<String, String> lastDateMap = getLastDateInfo();
        String roleCode = sysRoleMapper.getRoleCodeByUserId(sysUser.getId());

        log.info(roleCode + ":" + sysUser.getRealname() + "有效率排行榜接口开始");

        List<UserDto> userRoleList = new ArrayList<>();
        BigDecimal rate = BigDecimal.ZERO; //提成比例
        if (roleCode.equals("plan")) {
            userRoleList = sysRoleMapper.getUserByRoleCode("plan"); //编导
            rate = new BigDecimal("0.4");
        } else if (roleCode.equals("shot")) {
            userRoleList = sysRoleMapper.getUserByRoleCode("shot"); //拍摄
            rate = new BigDecimal("0.1");
        } else if (roleCode.equals("clip")) {
            userRoleList = sysRoleMapper.getUserByRoleCode("clip"); //剪辑
            rate = new BigDecimal("0.3");
        }

        if (userRoleList != null) {
            for (UserDto userDto : userRoleList) {
                //有效视频数量
                int effiVideoCount = userEfficientVideoMapMapper.getEffiVideoCountByUserId(2, Integer.parseInt(lastDateMap.get("year")), Integer.parseInt(lastDateMap.get("quarter")), userDto.getUserId(), null);
                //总视频数量
                int videoCount = userEfficientVideoMapMapper.getQuarterMaterialCountByUserId(roleCode, startDate, endNowDate, userDto.getUserId());
                //视频有效率
                BigDecimal effiRate = BigDecimal.ZERO;
                if (effiVideoCount != 0 && videoCount != 0) {
                    effiRate = new BigDecimal(String.valueOf(effiVideoCount)).divide(new BigDecimal(String.valueOf(videoCount))).multiply(new BigDecimal("100")).setScale(2, BigDecimal.ROUND_HALF_UP);
                }
                userDto.setEffiRate(effiRate);
            }
            Collections.sort(userRoleList, new EffiRateComparator());

            //获取剪辑人数并按照提点比例计算每个梯度的人数
            int userRoleLength = userRoleList.size();
            //获取前30%和中间50%的人数（视频有效率四舍五入）
            int userTopSize = new BigDecimal(String.valueOf(userRoleLength)).multiply(new BigDecimal("0.3")).setScale(0, BigDecimal.ROUND_HALF_UP).intValue();
            int userMiddleSize = new BigDecimal(String.valueOf(userRoleLength)).multiply(new BigDecimal("0.5")).setScale(0, BigDecimal.ROUND_HALF_UP).intValue();
            log.info(roleCode + "的总人数为：" + userRoleLength + ";前30%人数为：" + userTopSize + "；中间50%的人数为：" + userMiddleSize);

            int index = 0;
            for (int i = 0; i <= userRoleLength; i++) {
                index = index + 1;
                if (userRoleList.get(i).getUserId().equals(sysUser.getId())) {
                    log.info("当前登陆人userId:" + sysUser.getId() + ";姓名:" + sysUser.getRealname() + ";排名:" + index);
                    break;
                }
            }

            //总消耗--获取这个人素材总消耗
            //总绩效=总消耗*提点比例（5‰）*提成比例（剪辑30%）
            BigDecimal performance = BigDecimal.ZERO; //绩效
            BigDecimal totalCost = BigDecimal.ZERO;  //总消耗
            if (roleCode.equals("plan")) {
                totalCost = userEfficientVideoMapMapper.getTotalCostByPlanId(sysUser.getId(), startDate, endNowDate);
                if (index <= userTopSize) {
                    performance = totalCost.multiply(new BigDecimal("0.005")).multiply(new BigDecimal("0.4"));
                } else if (index > userTopSize && index <= (userTopSize + userMiddleSize)) {
                    performance = totalCost.multiply(new BigDecimal("0.003")).multiply(new BigDecimal("0.4"));
                } else {
                    performance = BigDecimal.ZERO;
                }
            } else if (roleCode.equals("shot")) {
                totalCost = userEfficientVideoMapMapper.getTotalCostByShotId(sysUser.getId(), startDate, endNowDate);
                if (index <= userTopSize) {
                    performance = totalCost.multiply(new BigDecimal("0.005")).multiply(new BigDecimal("0.1"));
                } else if (index > userTopSize && index <= (userTopSize + userMiddleSize)) {
                    performance = totalCost.multiply(new BigDecimal("0.003")).multiply(new BigDecimal("0.1"));
                } else {
                    performance = BigDecimal.ZERO;
                }
            } else {
                totalCost = userEfficientVideoMapMapper.getTotalCostByClipId(sysUser.getId(), startDate, endNowDate);
                if (index <= userTopSize) {
                    performance = totalCost.multiply(new BigDecimal("0.005")).multiply(new BigDecimal("0.3"));
                } else if (index > userTopSize && index <= (userTopSize + userMiddleSize)) {
                    performance = totalCost.multiply(new BigDecimal("0.003")).multiply(new BigDecimal("0.3"));
                } else {
                    performance = BigDecimal.ZERO;
                }
            }

            //去掉list中前30的数据
            if (size != null) {
                userRoleList.subList(size, userRoleList.size()).clear();
            }

            resultMap.put("userRoleList", userRoleList);
            resultMap.put("performance", performance);
        } else {
            resultMap.put("userRoleList", null);
            resultMap.put("performance", 0);
        }

        log.info(roleCode + ":" + sysUser.getRealname() + "有效率排行榜接口结束");
        return resultMap;
    }

    //3
    //素材、素材跑量、封面点击数、封面曝光数、行为数
    //{"videoUrl":"http://dadfadfadf.mp4","videoCost":34.56,"pageClickCount":1200,"pageExposureCount":3939,"behaviorCount":5000}
    public List<DesignMaterialInfo> materialInfoList() {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        //LoginUser sysUser = new LoginUser();
        //sysUser.setId("9712328fddd1443faf566ddb87693658");
        String roleCode = sysRoleMapper.getRoleCodeByUserId(sysUser.getId());

        log.info(roleCode + ":" + sysUser.getRealname() + "materialInfoList接口开始");

        Date date = new Date();
        Date getDate = DateUtils.addDay(date, -1);
        String startDate = DateUtils.getQuarterStartDate(getDate);
        String endDate = DateUtils.getQuarterEndDate(getDate);
        String endNowDate = DateUtils.formatDate(getDate);

        List<DesignMaterialInfo> materialInfoList = userEfficientVideoMapMapper.designMaterialInfoList(roleCode, sysUser.getId(), startDate, endNowDate);

        log.info(roleCode + ":" + sysUser.getRealname() + "materialInfoList接口结束");
        return materialInfoList;
    }

    //4
    //点击+符号 投放账户、投放账户、跑量
    //{"videoAccount":"aabbcc","servingAccount":"ttyyhh","cost":34234.43}
    public List<DesignMaterialInfo> materialAccountInfo(String materialInfoId) {
        log.info("materialAccountInfo接口开始");
        Map<String, Integer> quarterMap = DateUtils.lastQuarter(new Date());
        int year = quarterMap.get("year");
        int quarter = quarterMap.get("quarter");

        Map<String, String> quarterDate = DateUtils.quarterStartEndDate(year, quarter);
        String startDate = quarterDate.get("startDate");
        String endDate = quarterDate.get("endDate");

        List<DesignMaterialInfo> materialInfoList = userEfficientVideoMapMapper.getMaterialAccountInfoByMaterialId(materialInfoId, startDate, endDate);
        log.info("materialAccountInfo接口结束");
        return materialInfoList;
    }

    //获取前一天的年、季度信息
    private Map<String, String> getLastDateInfo() {
        Map<String, String> timeMap = new HashMap<String, String>();
        Date date = new Date();
        Date getDate = DateUtils.addDay(date, -1);
        String startDate = DateUtils.getQuarterStartDate(getDate);
        String endDate = DateUtils.getQuarterEndDate(getDate);
        String year = DateUtils.getYear(getDate) + "";
        String quarter = DateUtils.getQuarter(getDate) + "";

        timeMap.put("startDate", startDate);
        timeMap.put("endDate", endDate);
        timeMap.put("year", year);
        timeMap.put("quarter", quarter);
        return timeMap;
    }

    //获取快手上个季度的季度有效数、素材总数、上个季度的绩效
    public Map<String, Object> kuaishouDesignerLastQuarterPerformanceInfo() {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();

        //TODO 测试使用
        //LoginUser sysUser = new LoginUser();
        //sysUser.setId("f1f4c325a1234827af28fb4d957ffd70");
        //

        Map<String, Integer> quarterMap = DateUtils.lastQuarter(new Date());
        int year = quarterMap.get("year");
        int quarter = quarterMap.get("quarter");

        Map<String, String> quarterDate = DateUtils.quarterStartEndDate(year, quarter);
        String startDate = quarterDate.get("startDate");
        String endDate = quarterDate.get("endDate");
        SysRole role = userEfficientVideoMapMapper.getRoleIdByUserId(sysUser.getId());

        //有效视频数量 1toutiao 2kuaishou
        int effiVideoCount = userEfficientVideoMapMapper.getEffiVideoCountByUserId(2, year, quarter, sysUser.getId(), role.getId());
        //总视频数量
        int videoCount = userEfficientVideoMapMapper.getQuarterMaterialCountByUserId("plan", startDate, endDate, sysUser.getId());
        //获取季度总绩效
        BigDecimal performance = userEfficientVideoMapMapper.getQuarterPerformance(sysUser.getId(), year, quarter, 2);

        //素材列表
        List<DesignMaterialInfo> materialInfoList = userEfficientVideoMapMapper.designMaterialInfoList(role.getRoleCode(), sysUser.getId(), startDate, endDate);

        Map<String, Object> resultMap = new HashMap();
        resultMap.put("effiVideoCount", effiVideoCount);
        resultMap.put("videoCount", videoCount);
        resultMap.put("performance", performance);
        resultMap.put("materialInfoList", materialInfoList);

        return resultMap;
    }

    //TODO 获取头条上个季度的季度有效数、素材总数、上个季度的绩效
    public void toutiaoDesignerLastQuarterPerformanceInfo() {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        Map<String, Integer> quarterMap = DateUtils.lastQuarter(new Date());
        int year = quarterMap.get("year");
        int quarter = quarterMap.get("quarter");
        Map<String, String> quarterDate = DateUtils.quarterStartEndDate(year, quarter);
        String startDate = quarterDate.get("startDate");
        String endDate = quarterDate.get("endDate");
        SysRole role = userEfficientVideoMapMapper.getRoleIdByUserId(sysUser.getId());

        //有效视频数量 1toutiao 2kuaishou
        int effiVideoCount = userEfficientVideoMapMapper.getEffiVideoCountByUserId(2, year, quarter, sysUser.getId(), role.getId());
        //总视频数量
        int videoCount = userEfficientVideoMapMapper.getQuarterMaterialCountByUserId("plan", startDate, endDate, sysUser.getId());
        //获取季度总绩效
        BigDecimal performance = userEfficientVideoMapMapper.getQuarterPerformance(sysUser.getId(), year, quarter, 1);
    }

    //获取设计师组下所有人的绩效
    public Map<String, Object> designerLastQuarterPerformanceInfo() {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();

        //TODO 测试删除
        //LoginUser sysUser = new LoginUser();
        //sysUser.setId("825dd716d4c041f9ac1c600bf6183979");
        //

        Map<String, Integer> quarterMap = DateUtils.lastQuarter(new Date());
        int year = quarterMap.get("year");
        int quarter = quarterMap.get("quarter");
        //Map<String,String> quarterDate = DateUtils.quarterStartEndDate(year,quarter);
        //String startDate = quarterDate.get("startDate");
        //String endDate = quarterDate.get("endDate");
        //SysRole role = userEfficientVideoMapMapper.getRoleIdByUserId(sysUser.getId());

        String DepartId = userEfficientVideoMapMapper.getDepartIdByUserId(sysUser.getId());
        List<UserDto2> userDto2List = userEfficientVideoMapMapper.getUserListByDepartId(DepartId);

        //需要判断属于哪个部门
        //判断如果在parent_id是不是快手或者头条的
        int appType = 0; //头条1;快手2
        String parentId = userEfficientVideoMapMapper.getParentIdByDepartId(DepartId);
        if (parentId.equals("26c10ecb5e0d416bb01f8ff983f4588c")) {
            appType = 1;
        } else {
            appType = 2;
        }

        //循环获取userDto然后获取赋值季度、年度以及部门、绩效
        for (UserDto2 userDto : userDto2List) {
            userDto.setYear(year);
            userDto.setQuarter(quarter);
            String departName = userEfficientVideoMapMapper.getDepartNameByUserId(userDto.getUserId());
            //获取绩效
            BigDecimal performance = userEfficientVideoMapMapper.getClipQuarterCost(appType, quarter, year, userDto.getUserId());
            userDto.setDepartName(departName);
            userDto.setEffiRate(performance);
        }

        Map<String, Object> resultMap = new HashMap<>();
        resultMap.put("userList", userDto2List);

        return resultMap;
    }

    //平面设计师(快手、头条可能不一样，头条内广是单独的媒体不知道是怎么回事)
    public Map<String, Object> planeLastQuarterPerformanceInfo(int appType) {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();

        //测试删除
        //LoginUser sysUser = new LoginUser();
        //sysUser.setId("b0e4798234df4096936e6bc6d00db3ba");
        //

        Map<String, Integer> quarterMap = DateUtils.lastQuarter(new Date());
        int year = quarterMap.get("year");
        int quarter = quarterMap.get("quarter");
        //Map<String,String> quarterDate = DateUtils.quarterStartEndDate(year,quarter);
        List<PlaneProjectCostDTO> planeProjectCostList = userEfficientVideoMapMapper.getPlanePerformanceInfo(appType, year, quarter, sysUser.getId());
        BigDecimal performance = userEfficientVideoMapMapper.getQuarterPerformance(sysUser.getId(), year, quarter, appType);

        Map<String, Object> resultMap = new HashMap<>();
        resultMap.put("planeProjectCostList", planeProjectCostList);
        resultMap.put("performance", performance);

        return resultMap;
    }

    //获取同一媒体类型下上个季度所有人的绩效
    public Map<String, Object> quarterUserPerformanceList(Integer appType) {
        Map<String, Object> resultMap = new HashMap<>();

        //LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        Map<String, Integer> quarterMap = DateUtils.lastQuarter(new Date());
        int year = quarterMap.get("year");
        int quarter = quarterMap.get("quarter");

        List<UserDto2> userDtoList = userEfficientVideoMapMapper.getUserPerformanceList(appType, year, quarter);
        resultMap.put("userDtoList", userDtoList);

        return resultMap;
    }

    //获取快手绩效时时数据接口
    @Override
    public List<UserDto2> kuaishouPerformanceInfoNow() {
        //0 获取当前季度的时间
        Map<String, Integer> dateMap = DateUtils.getYearQuarter(new Date());
        Integer year = dateMap.get("year");
        Integer quarter = dateMap.get("quarter");
        //int year = 2019;
        //int quarter = 4;
        Map<String, String> timeMap = DateUtils.quarterStartEndDate(year, quarter);
        String startTime = timeMap.get("startDate");
        String endTime = timeMap.get("endDate");

        //1 获取快手下所有人
        List<UserDto2> userDto2List = userEfficientVideoMapMapper.getKuaishouDepartUserInfo();
        //2 循环所有人，并查询出每个人的有效视频数、总视频数、总消耗
        for (UserDto2 userDto : userDto2List) {
            String roleCode = userDto.getRoleCode();
            //2.1 获取有效视频数量
            Integer effiVideoCount = userEfficientVideoMapMapper.getEffiVideoCountByUserId(2, year, quarter, userDto.getUserId(), userDto.getRoleId());
            //2.2 获取有效视频总数
            Integer videoCount = userEfficientVideoMapMapper.getQuarterMaterialCountByUserId(userDto.getRoleCode(), startTime, endTime, userDto.getUserId());
            //2.3 总消耗
            BigDecimal cost = null;
            if ("clip".equals(roleCode)) {
                cost = userEfficientVideoMapMapper.getTotalCostByClipId(userDto.getUserId(), startTime, endTime);
            } else if ("plan".equals(roleCode)) {
                cost = userEfficientVideoMapMapper.getTotalCostByPlanId(userDto.getUserId(), startTime, endTime);
            } else if ("shot".equals(roleCode)) {
                cost = userEfficientVideoMapMapper.getTotalCostByShotId(userDto.getUserId(), startTime, endTime);
            }
            if(cost == null){
                cost = BigDecimal.ZERO;
            }
            userDto.setCost(cost);
            userDto.setVideoCount(videoCount);
            userDto.setEffiVideoCount(effiVideoCount);
            userDto.setAppType(2);
        }
        return userDto2List;
    }

    //获取快手优化师绩效时时数据接口
    @Override
    public List<UserDto2> kuaishouYouhuaPerformanceInfoNow() {
        //0 获取当前季度的时间
        Map<String, Integer> dateMap = DateUtils.getYearQuarter(new Date());
        Integer year = dateMap.get("year");
        Integer quarter = dateMap.get("quarter");
        //int year = 2019;
        //int quarter = 4;
        Map<String, String> timeMap = DateUtils.quarterStartEndDate(year, quarter);
        String startTime = timeMap.get("startDate");
        String endTime = timeMap.get("endDate");

        //1 获取快手下所有人
        List<UserDto2> userDto2List = userEfficientVideoMapMapper.getKuaishouYunyingDepartUserInfo();
        //2 循环所有人，并查询出每个人的有效视频数、总视频数、总消耗
        for (UserDto2 userDto : userDto2List) {
            BigDecimal cost = null;
            cost = userEfficientVideoMapMapper.getKuaishouYunyingTotalCost(userDto.getUserId(),2,startTime, endTime);
            if(cost == null){
                cost = BigDecimal.ZERO;
            }
            userDto.setCost(cost);
            userDto.setAppType(2);
        }
        return userDto2List;
    }

    //获取头条优化师绩效时时数据接口
    @Override
    public List<UserDto2> toutiaoYouhuaPerformanceInfoNow() {
        //0 获取当前季度的时间
        Map<String, Integer> dateMap = DateUtils.getYearQuarter(new Date());
        Integer year = dateMap.get("year");
        Integer quarter = dateMap.get("quarter");
        //int year = 2019;
        //int quarter = 4;
        Map<String, String> timeMap = DateUtils.quarterStartEndDate(year, quarter);
        String startTime = timeMap.get("startDate");
        String endTime = timeMap.get("endDate");

        //1 获取头条下所有优化师
        List<UserDto2> userDto2List = userEfficientVideoMapMapper.getToutiaoYunyingDepartUserInfo();
        //2 循环所有人，并查询出每个人的有效视频数、总视频数、总消耗
        for (UserDto2 userDto : userDto2List) {
            BigDecimal cost = null;
            cost = userEfficientVideoMapMapper.getToutiaoYunyingTotalCost(userDto.getUserId(),1,startTime, endTime);
            if(cost == null){
                cost = BigDecimal.ZERO;
            }
            userDto.setCost(cost);
            userDto.setAppType(1);
        }
        return userDto2List;
    }

    //获取快手优化师绩效周报
    @Override
    public List<UserDto2> kuaishouYouhuaWeekReport(String startDate, String endDate) {
        //1 获取快手下所有人
        List<UserDto2> userDto2List = userEfficientVideoMapMapper.getKuaishouYunyingDepartUserInfo();
        //2 循环所有人，并查询出每个优化师的总消耗
        for (UserDto2 userDto : userDto2List) {
            BigDecimal cost = null;
            cost = userEfficientVideoMapMapper.getKuaishouYunyingTotalCost(userDto.getUserId(),2,startDate, endDate);
            if(cost == null){
                cost = BigDecimal.ZERO;
            }
            userDto.setCost(cost);
            userDto.setAppType(2);
        }
        return userDto2List;
    }

    //获取头条优化师绩效周报
    @Override
    public List<UserDto2> toutiaoYouhuaWeekReport(String startDate, String endDate) {
        //1 获取快手下所有人
        List<UserDto2> userDto2List = userEfficientVideoMapMapper.getToutiaoYunyingDepartUserInfo();
        //2 循环所有人，并查询出每个人的有效视频数、总视频数、总消耗
        for (UserDto2 userDto : userDto2List) {
            BigDecimal cost = null;
            cost = userEfficientVideoMapMapper.getToutiaoYunyingTotalCost(userDto.getUserId(),1,startDate, endDate);
            if(cost == null){
                cost = BigDecimal.ZERO;
            }
            userDto.setCost(cost);
            userDto.setAppType(1);
        }
        return userDto2List;
    }

    //获取快手优化师所属账号的消耗等信息
    public List<AccountDTO> getKuaiShouYouhuaAccountsList(String userId, Integer appType, Integer year , Integer quarter){
        if(year == null || quarter == null){
            Map<String, Integer> dateMap = DateUtils.getYearQuarter(new Date());
            year = dateMap.get("year");
            quarter = dateMap.get("quarter");
        }
        Map<String, String> timeMap = DateUtils.quarterStartEndDate(year, quarter);
        String startTime = timeMap.get("startDate");
        String endTime = timeMap.get("endDate");
        return userEfficientVideoMapMapper.getKuaishouAccountTotalCost(userId, appType, startTime, endTime);
    }

    //获取头条优化师所属账号的消耗等信息
    @Override
    public List<AccountDTO> getToutiaoYouhuaAccountsList(String userId, Integer appType, Integer year, Integer quarter){
        if(year == null || quarter == null){
            Map<String, Integer> dateMap = DateUtils.getYearQuarter(new Date());
            year = dateMap.get("year");
            quarter = dateMap.get("quarter");
        }
        Map<String, String> timeMap = DateUtils.quarterStartEndDate(year, quarter);
        String startTime = timeMap.get("startDate");
        String endTime = timeMap.get("endDate");
        return userEfficientVideoMapMapper.getToutiaoAccountTotalCost(userId, appType, startTime, endTime);
    }

    //获取头条绩效实时数据
    public List<UserDto2> toutiaoPerformanceInfoNow() {
        //0 获取当前季度的时间
        Map<String, Integer> dateMap = DateUtils.getYearQuarter(new Date());
        Integer year = dateMap.get("year");
        Integer quarter = dateMap.get("quarter");
        Map<String, String> timeMap = DateUtils.quarterStartEndDate(year, quarter);
        String startTime = timeMap.get("startDate");
        String endTime = timeMap.get("endDate");

        //1 获取头条所有人
        List<UserDto2> userDto2List = userEfficientVideoMapMapper.getToutiaoDepartUserInfo();
        //2 循环所有人，并查询出每个人的有效视频数、总视频数、总消耗
        for (UserDto2 userDto : userDto2List) {
            String roleCode = userDto.getRoleCode();
            //2.1 获取有效视频数量
            Integer effiVideoCount = userEfficientVideoMapMapper.getEffiVideoCountByUserId(1, year, quarter, userDto.getUserId(), userDto.getRoleId());
            //2.2 获取视频总数
            Integer videoCount = userEfficientVideoMapMapper.getQuarterMaterialCountByUserId(userDto.getRoleCode(), startTime, endTime, userDto.getUserId());
            //2.3 总消耗
            BigDecimal cost = null;
            if ("clip".equals(roleCode)) {
                cost = userEfficientVideoMapMapper.toutiaoGetTotalCostByClipId(userDto.getUserId(), startTime, endTime);
            } else if ("plan".equals(roleCode)) {
                cost = userEfficientVideoMapMapper.toutiaoGetTotalCostByPlanId(userDto.getUserId(), startTime, endTime);
            } else if ("shot".equals(roleCode)) {
                cost = userEfficientVideoMapMapper.toutiaoGetTotalCostByShotId(userDto.getUserId(), startTime, endTime);
            }
            if(cost == null){
                cost = BigDecimal.ZERO;
            }
            userDto.setCost(cost);
            userDto.setVideoCount(videoCount);
            userDto.setEffiVideoCount(effiVideoCount);
            userDto.setAppType(1);
        }
        return userDto2List;
    }

    //快手绩效周报
    @Override
    public List<UserDto2> kuaishouWeekReport(String startDate, String endDate) {
        //1 获取快手下所有人
        List<UserDto2> userDto2List = userEfficientVideoMapMapper.getKuaishouDepartUserInfo();
        //2 循环所有人，并查询出每个人的有效视频数、总视频数、总消耗
        for (UserDto2 userDto : userDto2List) {
            String roleCode = userDto.getRoleCode();
            //2.1 获取有效视频数量
            Integer effiVideoCount = userEfficientVideoMapMapper.getEffiVideoCountByCalcDate(startDate, endDate, userDto.getRoleId(), userDto.getUserId(), 2);
            //2.2 获取有效视频总数
            Integer videoCount = userEfficientVideoMapMapper.getQuarterMaterialCountByUserId(userDto.getRoleCode(), startDate, endDate, userDto.getUserId());
            //2.3 总消耗
            BigDecimal cost = null;
            if ("clip".equals(roleCode)) {
                cost = userEfficientVideoMapMapper.getTotalCostByClipId(userDto.getUserId(), startDate, endDate);
            } else if ("plan".equals(roleCode)) {
                cost = userEfficientVideoMapMapper.getTotalCostByPlanId(userDto.getUserId(), startDate, endDate);
            } else if ("shot".equals(roleCode)) {
                cost = userEfficientVideoMapMapper.getTotalCostByShotId(userDto.getUserId(), startDate, endDate);
            }
            if(cost == null){
                cost = BigDecimal.ZERO;
            }
            userDto.setCost(cost);
            userDto.setVideoCount(videoCount);
            userDto.setEffiVideoCount(effiVideoCount);
        }

        return userDto2List;
    }

    //头条绩效周报
    public List<UserDto2> toutiaoWeekReport(String startDate, String endDate) {
        //1 获取头条所有人
        List<UserDto2> userDto2List = userEfficientVideoMapMapper.getToutiaoDepartUserInfo();
        //2 循环所有人，并查询出每个人的有效视频数、总视频数、总消耗
        for (UserDto2 userDto : userDto2List) {
            String roleCode = userDto.getRoleCode();
            //2.1 获取有效视频数量
            Integer effiVideoCount = userEfficientVideoMapMapper.getEffiVideoCountByCalcDate(startDate, endDate, userDto.getRoleId(), userDto.getUserId(), 1);
            //2.2 获取视频总数
            Integer videoCount = userEfficientVideoMapMapper.getQuarterMaterialCountByUserId(userDto.getRoleCode(), startDate, endDate, userDto.getUserId());
            //2.3 总消耗
            BigDecimal cost = null;
            if ("clip".equals(roleCode)) {
                cost = userEfficientVideoMapMapper.toutiaoGetTotalCostByClipId(userDto.getUserId(), startDate, endDate);
            } else if ("plan".equals(roleCode)) {
                cost = userEfficientVideoMapMapper.toutiaoGetTotalCostByPlanId(userDto.getUserId(), startDate, endDate);
            } else if ("shot".equals(roleCode)) {
                cost = userEfficientVideoMapMapper.toutiaoGetTotalCostByShotId(userDto.getUserId(), startDate, endDate);
            }

            if(cost == null){
                cost = BigDecimal.ZERO;
            }
            userDto.setCost(cost);
            userDto.setVideoCount(videoCount);
            userDto.setEffiVideoCount(effiVideoCount);
        }
        return userDto2List;
    }

    //快手季度绩效报表
    @Override
    public List<UserDto2> kuaishouQuarterReport(Integer year, Integer quarter) {
        //0 获取当季度的开始和结束时间
        Map<String, String> timeMap = DateUtils.quarterStartEndDate(year, quarter);
        String startTime = timeMap.get("startDate");
        String endTime = timeMap.get("endDate");

        //1 获取快手下所有人
        List<UserDto2> userDto2List = userEfficientVideoMapMapper.getKuaishouDepartAllUserInfo();
        //2 循环所有人，并查询出每个人的有效视频数、总视频数、总消耗
        for (UserDto2 userDto : userDto2List) {
            //String roleCode = userDto.getRoleCode();
            //2.1 获取有效视频数量
            Integer effiVideoCount = userEfficientVideoMapMapper.getEffiVideoCountByUserId(2, year, quarter, userDto.getUserId(), userDto.getRoleId());
            //2.2 获取有效视频总数
            Integer videoCount = userEfficientVideoMapMapper.getQuarterMaterialCountByUserId(userDto.getRoleCode(), startTime, endTime, userDto.getUserId());
            //2.3 总消耗
            BigDecimal cost = userEfficientVideoMapMapper.getQuarterCost(userDto.getUserId(), year, quarter, 2);
            if (cost == null) {
                cost = BigDecimal.ZERO;
            }
            //2.4 当季度绩效
            BigDecimal performance = userEfficientVideoMapMapper.getQuarterPerformance(userDto.getUserId(), year, quarter, 2);
            if(performance==null){
                performance=BigDecimal.ZERO;
            }
            userDto.setEffiRate(performance);
            userDto.setCost(cost);
            userDto.setVideoCount(videoCount);
            userDto.setEffiVideoCount(effiVideoCount);
        }
        return userDto2List;
    }

    //头条季度绩效报表
    @Override
    public List<UserDto2> toutiaoQuarterReport(Integer year, Integer quarter) {
        //0 获取当季度的开始和结束时间
        Map<String, String> timeMap = DateUtils.quarterStartEndDate(year, quarter);
        String startTime = timeMap.get("startDate");
        String endTime = timeMap.get("endDate");

        //1 获取头条下所有人
        List<UserDto2> userDto2List = userEfficientVideoMapMapper.getToutiaoDepartAllUserInfo();
        //2 循环所有人，并查询出每个人的有效视频数、总视频数、总消耗
        for (UserDto2 userDto : userDto2List) {
            //String roleCode = userDto.getRoleCode();
            //2.1 获取有效视频数量
            Integer effiVideoCount = userEfficientVideoMapMapper.getEffiVideoCountByUserId(1, year, quarter, userDto.getUserId(), userDto.getRoleId());
            //2.2 获取有效视频总数
            Integer videoCount = userEfficientVideoMapMapper.getQuarterMaterialCountByUserId(userDto.getRoleCode(), startTime, endTime, userDto.getUserId());
            //2.3 总消耗
            BigDecimal cost = userEfficientVideoMapMapper.getQuarterCost(userDto.getUserId(), year, quarter, 1);
            if (cost == null) {
                cost = BigDecimal.ZERO;
            }
            //2.4 当季度绩效
            BigDecimal performance = userEfficientVideoMapMapper.getQuarterPerformance(userDto.getUserId(), year, quarter, 1);
            if(performance==null){
                performance=BigDecimal.ZERO;
            }
            userDto.setEffiRate(performance);
            userDto.setCost(cost);
            userDto.setVideoCount(videoCount);
            userDto.setEffiVideoCount(effiVideoCount);
        }
        return userDto2List;
    }

    //根据userid,appType,年度,季度查询有效视频列表
    public List<MaterialInfo> effiVideoList(Integer appType, Integer year, Integer quarter, String userId){
        if(year == null || quarter == null){
            Map<String, Integer> dateMap = DateUtils.getYearQuarter(new Date());
            year = dateMap.get("year");
            quarter = dateMap.get("quarter");
        }
        List<MaterialInfo> materialInfoList = userEfficientVideoMapMapper.getEffiVideoList(appType, year, quarter, userId);
        return materialInfoList;
    }

    //根据userid,appType,年度,季度查询有效视频列表
    public List<MaterialInfo> getEffiWeekVideoList(Integer appType, String startDate, String endDate, String userId){
        List<MaterialInfo> materialInfoList = userEfficientVideoMapMapper.getEffiWeekVideoList(appType, startDate, endDate, userId);
        return materialInfoList;
    }

    //快手优化季度绩效报表
    @Override
    public List<UserDto2> kuaishouYouhuaQuarterReport(Integer year, Integer quarter) {
        //1 获取快手下所有人
        List<UserDto2> userDto2List = userEfficientVideoMapMapper.getKuaishouYunyingDepartUserInfo();
        //2 循环所有人，并查询出每个人的有效视频数、总视频数、总消耗
        for (UserDto2 userDto : userDto2List) {
            //总消耗
            BigDecimal cost = userEfficientVideoMapMapper.getQuarterCost(userDto.getUserId(), year, quarter, 2);
            if (cost == null) {
                cost = BigDecimal.ZERO;
            }
            //当季度绩效
            BigDecimal performance = userEfficientVideoMapMapper.getQuarterPerformance(userDto.getUserId(), year, quarter, 2);
            if(performance==null){
                performance=BigDecimal.ZERO;
            }
            userDto.setEffiRate(performance);
            userDto.setCost(cost);
            userDto.setYear(year);
            userDto.setQuarter(quarter);
            userDto.setAppType(2);
        }
        return userDto2List;
    }

    //头条优化季度绩效报表
    //@Override
    public List<UserDto2> toutiaoYouhuaQuarterReport(Integer year, Integer quarter) {
        //1 获取快手下所有人
        List<UserDto2> userDto2List = userEfficientVideoMapMapper.getToutiaoYunyingDepartUserInfo();
        //2 循环所有人，并查询出每个人的有效视频数、总视频数、总消耗
        for (UserDto2 userDto : userDto2List) {
            //总消耗
            BigDecimal cost = userEfficientVideoMapMapper.getQuarterCost(userDto.getUserId(), year, quarter, 1);
            if (cost == null) {
                cost = BigDecimal.ZERO;
            }
            //当季度绩效
            BigDecimal performance = userEfficientVideoMapMapper.getQuarterPerformance(userDto.getUserId(), year, quarter, 1);
            if(performance==null){
                performance=BigDecimal.ZERO;
            }
            userDto.setEffiRate(performance);
            userDto.setCost(cost);
            userDto.setYear(year);
            userDto.setQuarter(quarter);
            userDto.setAppType(1);
        }
        return userDto2List;
    }

}

