package cn.com.ctop.crawler.modules.appium.controller;

import cn.com.ctop.crawler.modules.appium.entity.AppiumTask;
import cn.com.ctop.crawler.modules.appium.service.IAppiumTaskService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 任务表
 * @author jeecg-boot
 * @date 2019-10-31
 * @version V1.0
 */
@Slf4j
@Api(tags = "任务表")
@RestController
@RequestMapping("/appium/appiumTask")
public class AppiumTaskController {
    @Autowired
    private IAppiumTaskService appiumTaskService;

    /**
     * 分页列表查询
     *
     * @param appiumTask
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "任务表-分页列表查询")
    @ApiOperation(value = "任务表-分页列表查询", notes = "任务表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<AppiumTask>> queryPageList(AppiumTask appiumTask,
                                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                   HttpServletRequest req) {
        Result<IPage<AppiumTask>> result = new Result<IPage<AppiumTask>>();
        QueryWrapper<AppiumTask> queryWrapper = QueryGenerator.initQueryWrapper(appiumTask, req.getParameterMap());
        Page<AppiumTask> page = new Page<AppiumTask>(pageNo, pageSize);
        IPage<AppiumTask> pageList = appiumTaskService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param appiumTask
     * @return
     */
    @AutoLog(value = "任务表-添加")
    @ApiOperation(value = "任务表-添加", notes = "任务表-添加")
    @PostMapping(value = "/add")
    public Result<AppiumTask> add(@RequestBody AppiumTask appiumTask) {
        Result<AppiumTask> result = new Result<AppiumTask>();
        try {
            appiumTaskService.save(appiumTask);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param appiumTask
     * @return
     */
    @AutoLog(value = "任务表-编辑")
    @ApiOperation(value = "任务表-编辑", notes = "任务表-编辑")
    @PutMapping(value = "/edit")
    public Result<AppiumTask> edit(@RequestBody AppiumTask appiumTask) {
        Result<AppiumTask> result = new Result<AppiumTask>();
        AppiumTask appiumTaskEntity = appiumTaskService.getById(appiumTask.getId());
        if (appiumTaskEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = appiumTaskService.updateById(appiumTask);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "任务表-通过id删除")
    @ApiOperation(value = "任务表-通过id删除", notes = "任务表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            appiumTaskService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "任务表-批量删除")
    @ApiOperation(value = "任务表-批量删除", notes = "任务表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<AppiumTask> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<AppiumTask> result = new Result<AppiumTask>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.appiumTaskService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "任务表-通过id查询")
    @ApiOperation(value = "任务表-通过id查询", notes = "任务表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<AppiumTask> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<AppiumTask> result = new Result<AppiumTask>();
        AppiumTask appiumTask = appiumTaskService.getById(id);
        if (appiumTask == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(appiumTask);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<AppiumTask> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                AppiumTask appiumTask = JSON.parseObject(deString, AppiumTask.class);
                queryWrapper = QueryGenerator.initQueryWrapper(appiumTask, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<AppiumTask> pageList = appiumTaskService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "任务表列表");
        mv.addObject(NormalExcelConstants.CLASS, AppiumTask.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("任务表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    @RequestMapping(value = "/run/login", method = RequestMethod.POST)
    public Result<?> runLogin(HttpServletRequest request, HttpServletResponse response, Integer taskId, Integer deviceId) {
        Thread thread = new Thread() {
            @Override
            public void run() {
                appiumTaskService.runKuaishouLoginTask();
            }
        };
        thread.start();
        return Result.ok("任务已启动");
    }
    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<AppiumTask> listAppiumTasks = ExcelImportUtil.importExcel(file.getInputStream(), AppiumTask.class, params);
                appiumTaskService.saveBatch(listAppiumTasks);
                return Result.ok("文件导入成功！数据行数:" + listAppiumTasks.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
