package cn.com.ctop.crawler.modules.appium.controller;

import cn.com.ctop.crawler.modules.appium.entity.AppiumTaskItem;
import cn.com.ctop.crawler.modules.appium.service.IAppiumTaskItemService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 任务内容表
 * @author jeecg-boot
 * @date 2019-10-31
 * @version V1.0
 */
@Slf4j
@Api(tags = "任务内容表")
@RestController
@RequestMapping("/appium/appiumTaskItem")
public class AppiumTaskItemController {
    @Autowired
    private IAppiumTaskItemService appiumTaskItemService;

    /**
     * 分页列表查询
     *
     * @param appiumTaskItem
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "任务内容表-分页列表查询")
    @ApiOperation(value = "任务内容表-分页列表查询", notes = "任务内容表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<AppiumTaskItem>> queryPageList(AppiumTaskItem appiumTaskItem,
                                                       @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                       @RequestParam(name = "pageSize", defaultValue = "100") Integer pageSize,
                                                       HttpServletRequest req) {
        Result<IPage<AppiumTaskItem>> result = new Result<IPage<AppiumTaskItem>>();
        QueryWrapper<AppiumTaskItem> queryWrapper = QueryGenerator.initQueryWrapper(appiumTaskItem, req.getParameterMap());
        Page<AppiumTaskItem> page = new Page<AppiumTaskItem>(pageNo, pageSize);
        IPage<AppiumTaskItem> pageList = appiumTaskItemService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param appiumTaskItem
     * @return
     */
    @AutoLog(value = "任务内容表-添加")
    @ApiOperation(value = "任务内容表-添加", notes = "任务内容表-添加")
    @PostMapping(value = "/add")
    public Result<AppiumTaskItem> add(@RequestBody AppiumTaskItem appiumTaskItem) {
        Result<AppiumTaskItem> result = new Result<AppiumTaskItem>();
        try {
            appiumTaskItemService.save(appiumTaskItem);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param appiumTaskItem
     * @return
     */
    @AutoLog(value = "任务内容表-编辑")
    @ApiOperation(value = "任务内容表-编辑", notes = "任务内容表-编辑")
    @PutMapping(value = "/edit")
    public Result<AppiumTaskItem> edit(@RequestBody AppiumTaskItem appiumTaskItem) {
        Result<AppiumTaskItem> result = new Result<AppiumTaskItem>();
        AppiumTaskItem appiumTaskItemEntity = appiumTaskItemService.getById(appiumTaskItem.getId());
        if (appiumTaskItemEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = appiumTaskItemService.updateById(appiumTaskItem);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "任务内容表-通过id删除")
    @ApiOperation(value = "任务内容表-通过id删除", notes = "任务内容表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            appiumTaskItemService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "任务内容表-批量删除")
    @ApiOperation(value = "任务内容表-批量删除", notes = "任务内容表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<AppiumTaskItem> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<AppiumTaskItem> result = new Result<AppiumTaskItem>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.appiumTaskItemService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "任务内容表-通过id查询")
    @ApiOperation(value = "任务内容表-通过id查询", notes = "任务内容表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<AppiumTaskItem> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<AppiumTaskItem> result = new Result<AppiumTaskItem>();
        AppiumTaskItem appiumTaskItem = appiumTaskItemService.getById(id);
        if (appiumTaskItem == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(appiumTaskItem);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<AppiumTaskItem> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                AppiumTaskItem appiumTaskItem = JSON.parseObject(deString, AppiumTaskItem.class);
                queryWrapper = QueryGenerator.initQueryWrapper(appiumTaskItem, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<AppiumTaskItem> pageList = appiumTaskItemService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "任务内容表列表");
        mv.addObject(NormalExcelConstants.CLASS, AppiumTaskItem.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("任务内容表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<AppiumTaskItem> listAppiumTaskItems = ExcelImportUtil.importExcel(file.getInputStream(), AppiumTaskItem.class, params);
                appiumTaskItemService.saveBatch(listAppiumTaskItems);
                return Result.ok("文件导入成功！数据行数:" + listAppiumTaskItems.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
