package cn.com.ctop.crawler.modules.appium.controller;

import cn.com.ctop.crawler.modules.appium.entity.AppiumTaskLog;
import cn.com.ctop.crawler.modules.appium.service.IAppiumTaskLogService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 任务日志表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2019-11-01
 */
@Slf4j
@Api(tags = "任务日志表")
@RestController
@RequestMapping("/appium/appiumTaskLog")
public class AppiumTaskLogController {
    @Autowired
    private IAppiumTaskLogService appiumTaskLogService;

    /**
     * 分页列表查询
     *
     * @param appiumTaskLog
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "任务日志表-分页列表查询")
    @ApiOperation(value = "任务日志表-分页列表查询", notes = "任务日志表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<AppiumTaskLog>> queryPageList(AppiumTaskLog appiumTaskLog,
                                                      @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                      @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                      HttpServletRequest req) {
        Result<IPage<AppiumTaskLog>> result = new Result<>();
        QueryWrapper<AppiumTaskLog> queryWrapper = QueryGenerator.initQueryWrapper(appiumTaskLog, req.getParameterMap());
        Page<AppiumTaskLog> page = new Page<>(pageNo, pageSize);
        IPage<AppiumTaskLog> pageList = appiumTaskLogService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param appiumTaskLog
     * @return
     */
    @AutoLog(value = "任务日志表-添加")
    @ApiOperation(value = "任务日志表-添加", notes = "任务日志表-添加")
    @PostMapping(value = "/add")
    public Result<AppiumTaskLog> add(@RequestBody AppiumTaskLog appiumTaskLog) {
        Result<AppiumTaskLog> result = new Result<>();
        try {
            appiumTaskLogService.save(appiumTaskLog);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param appiumTaskLog
     * @return
     */
    @AutoLog(value = "任务日志表-编辑")
    @ApiOperation(value = "任务日志表-编辑", notes = "任务日志表-编辑")
    @PutMapping(value = "/edit")
    public Result<AppiumTaskLog> edit(@RequestBody AppiumTaskLog appiumTaskLog) {
        Result<AppiumTaskLog> result = new Result<>();
        AppiumTaskLog appiumTaskLogEntity = appiumTaskLogService.getById(appiumTaskLog.getId());
        if (appiumTaskLogEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = appiumTaskLogService.updateById(appiumTaskLog);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "任务日志表-通过id删除")
    @ApiOperation(value = "任务日志表-通过id删除", notes = "任务日志表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            appiumTaskLogService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "任务日志表-批量删除")
    @ApiOperation(value = "任务日志表-批量删除", notes = "任务日志表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<AppiumTaskLog> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<AppiumTaskLog> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.appiumTaskLogService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "任务日志表-通过id查询")
    @ApiOperation(value = "任务日志表-通过id查询", notes = "任务日志表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<AppiumTaskLog> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<AppiumTaskLog> result = new Result<>();
        AppiumTaskLog appiumTaskLog = appiumTaskLogService.getById(id);
        if (appiumTaskLog == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(appiumTaskLog);
            result.setSuccess(true);
        }
        return result;
    }
}
