package cn.com.ctop.kuaishou.modules.batch.controller;

import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouActionBarText;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouCampaignCreate;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouActionBarTextService;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouCampaignCreateService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @Description: 快手-行动号召文案
 * @Author: jeecg-boot
 * @Date: 2019-08-15
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "快手-行动号召文案")
@RestController
@RequestMapping("/kuaishou/kuaiShouActionBarText")
public class KuaiShouActionBarTextController {
    @Autowired
    private IKuaiShouActionBarTextService kuaiShouActionBarTextService;
    @Autowired
    private IKuaiShouCampaignCreateService campaignCreateService;

    /**
     * 分页列表查询
     *
     * @param kuaiShouActionBarText
     * @param pageNo
     * @param pageSize
     * @return
     */
    @AutoLog(value = "快手-行动号召文案-分页列表查询")
    @ApiOperation(value = "快手-行动号召文案-分页列表查询", notes = "快手-行动号召文案-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaiShouActionBarText>> queryPageList(KuaiShouActionBarText kuaiShouActionBarText,
                                                              @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                              @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                              @RequestParam(name = "campaignId") String campaignId) {
        Result<IPage<KuaiShouActionBarText>> result = new Result<IPage<KuaiShouActionBarText>>();
        KuaiShouCampaignCreate campaignCreate = campaignCreateService.selectCampaignByCampaignId(campaignId);
        if (!Check.isNull(campaignCreate)) {
            QueryWrapper<KuaiShouActionBarText> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("type", campaignCreate.getType());
            Page<KuaiShouActionBarText> page = new Page<KuaiShouActionBarText>(pageNo, pageSize);
            IPage<KuaiShouActionBarText> pageList = kuaiShouActionBarTextService.page(page, queryWrapper);
            result.setSuccess(true);
            result.setResult(pageList);
        } else {
            result.setSuccess(false);
            result.setMessage("未获取到广告计划信息");
        }

        return result;
    }

    /**
     * 添加
     *
     * @param kuaiShouActionBarText
     * @return
     */
    @AutoLog(value = "快手-行动号召文案-添加")
    @ApiOperation(value = "快手-行动号召文案-添加", notes = "快手-行动号召文案-添加")
    @PostMapping(value = "/add")
    public Result<KuaiShouActionBarText> add(@RequestBody KuaiShouActionBarText kuaiShouActionBarText) {
        Result<KuaiShouActionBarText> result = new Result<KuaiShouActionBarText>();
        try {
            kuaiShouActionBarTextService.save(kuaiShouActionBarText);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaiShouActionBarText
     * @return
     */
    @AutoLog(value = "快手-行动号召文案-编辑")
    @ApiOperation(value = "快手-行动号召文案-编辑", notes = "快手-行动号召文案-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaiShouActionBarText> edit(@RequestBody KuaiShouActionBarText kuaiShouActionBarText) {
        Result<KuaiShouActionBarText> result = new Result<KuaiShouActionBarText>();
        KuaiShouActionBarText kuaiShouActionBarTextEntity = kuaiShouActionBarTextService.getById(kuaiShouActionBarText.getId());
        if (kuaiShouActionBarTextEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaiShouActionBarTextService.updateById(kuaiShouActionBarText);
            //TODO 返回false说明什么？
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手-行动号召文案-通过id删除")
    @ApiOperation(value = "快手-行动号召文案-通过id删除", notes = "快手-行动号召文案-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaiShouActionBarTextService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "快手-行动号召文案-批量删除")
    @ApiOperation(value = "快手-行动号召文案-批量删除", notes = "快手-行动号召文案-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaiShouActionBarText> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaiShouActionBarText> result = new Result<KuaiShouActionBarText>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaiShouActionBarTextService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手-行动号召文案-通过id查询")
    @ApiOperation(value = "快手-行动号召文案-通过id查询", notes = "快手-行动号召文案-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaiShouActionBarText> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaiShouActionBarText> result = new Result<KuaiShouActionBarText>();
        KuaiShouActionBarText kuaiShouActionBarText = kuaiShouActionBarTextService.getById(id);
        if (kuaiShouActionBarText == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaiShouActionBarText);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaiShouActionBarText> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaiShouActionBarText kuaiShouActionBarText = JSON.parseObject(deString, KuaiShouActionBarText.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouActionBarText, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaiShouActionBarText> pageList = kuaiShouActionBarTextService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "快手-行动号召文案列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaiShouActionBarText.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("快手-行动号召文案列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaiShouActionBarText> listKuaiShouActionBarTexts = ExcelImportUtil.importExcel(file.getInputStream(), KuaiShouActionBarText.class, params);
                kuaiShouActionBarTextService.saveBatch(listKuaiShouActionBarTexts);
                return Result.ok("文件导入成功！数据行数:" + listKuaiShouActionBarTexts.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
