package cn.com.ctop.kuaishou.modules.batch.service;

import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouCampaign;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouCreative;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouGroup;
import com.alibaba.fastjson.JSONObject;

import java.math.BigDecimal;

public interface IBatchService {
    /**
     * 获取 总花费
     *
     * @param accountId
     * @return
     */
    BigDecimal getCost(Long accountId);

    /**
     * 获取总计划预算
     *
     * @param accountId
     * @return
     */
    BigDecimal getBudget(Long accountId);

    /**
     * 获取计划信息
     *
     * @param accountId
     * @param campaignId
     * @return
     */
    KuaiShouCampaign getCampaignInfo(Long accountId, Long campaignId);


    /**
     * 获取项目最高出价
     *
     * @param accountId
     * @return
     */
    BigDecimal getProjectMaxBid(Long accountId);

    /**
     * 校验广告组名称是否重复
     *
     * @param campaignId
     * @param unitName
     * @return
     */
    Boolean checkUnitName(Long campaignId, String unitName);

    /**
     * 批量创建广告组
     *
     * @param requestJson
     * @return
     */
    JSONObject createUnit(JSONObject requestJson) throws Exception;


    /**
     * 获取广告组信息
     *
     * @param accountId
     * @param unitId
     * @return
     */
    KuaiShouGroup getUnitInfo(Long accountId, Long unitId);

    /**
     * 创建创意
     *
     * @param requestJson
     * @return
     */
    JSONObject createCreative(JSONObject requestJson) throws Exception;

    /**
     * 获取创意信息
     *
     * @param accountId
     * @param creativeId
     * @return
     */
    KuaiShouCreative getCreativeInfo(Long accountId, Long creativeId);

    /**
     * 修改计划信息
     *
     * @param requestJson
     * @return
     */
    JSONObject updateCampaign(JSONObject requestJson) throws Exception;


    /**
     * 修改广告组
     *
     * @param requestJson
     * @return
     */
    JSONObject updateUnit(JSONObject requestJson) throws Exception;

    /**
     * 修改广告创意
     *
     * @param requestJson
     * @return
     */
    JSONObject updateCreative(JSONObject requestJson) throws Exception;
}
